@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -30,6 +30,7 @@
 
 #include <libanjuta/resources.h>
 #include <libanjuta/anjuta-debug.h>
+#include <libanjuta/anjuta-utils.h>
 #include <libanjuta/interfaces/ianjuta-file-savable.h>
 #include <libanjuta/interfaces/ianjuta-project-manager.h>
 #include <libanjuta/interfaces/ianjuta-terminal.h>
@@ -37,6 +38,9 @@
 
 #include "executer.h"
 
+#define PREF_USE_SB "build.use_scratchbox"
+#define PREF_SB_PATH "build.scratchbox.path"
+
 static gboolean
 get_program_parameters (BasicAutotoolsPlugin *plugin,
 						const gchar *pre_select_uri,
@@ -67,7 +71,11 @@ get_program_parameters (BasicAutotoolsPlugin *plugin,
 							 IANJUTA_PROJECT_MANAGER_TARGET_EXECUTABLE,
 											 NULL);
 		if (!exec_targets)
+		{
+			anjuta_util_dialog_error(GTK_WINDOW (ANJUTA_PLUGIN(plugin)->shell),
+															 _("No executables in this project!"));
 			return FALSE;
+		}
 	}
 	else
 	{
@@ -211,6 +219,7 @@ get_program_parameters (BasicAutotoolsPlugin *plugin,
 void
 execute_program (BasicAutotoolsPlugin* plugin, const gchar *pre_select_uri)
 {
+	AnjutaPreferences* prefs = anjuta_shell_get_preferences (ANJUTA_PLUGIN(plugin)->shell, NULL);
 	gboolean run_in_terminal, error_condition;
 	gchar *target = NULL, *args = NULL, *dir = NULL, *cmd = NULL;
 
@@ -243,7 +252,7 @@ execute_program (BasicAutotoolsPlugin* plugin, const gchar *pre_select_uri)
 				error_condition = TRUE;
 				target = NULL;
 				anjuta_util_dialog_error (GTK_WINDOW (ANJUTA_PLUGIN (plugin)->shell),
-										  _("No file or Project opened."));
+										  _("No file or project currently opened."));
 			}
 			else
 			{
@@ -357,14 +366,25 @@ execute_program (BasicAutotoolsPlugin* plugin, const gchar *pre_select_uri)
 		return;
 	}
 	
+	
 	if (args && strlen (args) > 0)
 		cmd = g_strconcat (target, " ", args, NULL);
 	else
 		cmd = g_strdup (target);
-	
-	DEBUG_PRINT ("Command is: %s", cmd);
 
-	dir = g_path_get_dirname (target);
+	if (anjuta_preferences_get_int (prefs , PREF_USE_SB))
+	{
+		const gchar* sb_path = anjuta_preferences_get(prefs, PREF_SB_PATH);
+		/* we need to skip the /scratchbox/users part, maybe could be done more clever */
+		const gchar* real_dir = strstr(target, "/home");
+		gchar* oldcmd = cmd;
+		cmd = g_strdup_printf("%s/login -d %s \"%s\"", sb_path,
+									  real_dir, oldcmd);
+		g_free(oldcmd);
+		dir = g_strdup(real_dir);
+	}
+	else
+		dir = g_path_get_dirname (target);
 	if (run_in_terminal)
 	{
 		IAnjutaTerminal *term;
