@@ -111,7 +111,11 @@ struct _BreakpointsDBase
 	GtkWidget *disableall_button;
 
 	/* Menu action */
-	GtkActionGroup *action_group;
+	GtkActionGroup *debugger_group;
+	GtkActionGroup *permanent_group;
+
+	/* Editor watch id */
+	guint editor_watch;
 };
 
 
@@ -130,29 +134,6 @@ enum {
 /* Helper functions
  *---------------------------------------------------------------------------*/
 
-static IAnjutaEditor *
-get_current_editor (DebugManagerPlugin *plugin)
-{
-	IAnjutaDocumentManager *docman;
-	IAnjutaEditor* ed = NULL;
-
-	docman = IANJUTA_DOCUMENT_MANAGER (anjuta_shell_get_object (ANJUTA_PLUGIN (plugin)->shell,
-					"IAnjutaDocumentManager", NULL));
-									   
-	if (docman != NULL)
-	{
-		IAnjutaDocument* doc;
-		
-		doc = ianjuta_document_manager_get_current_document(docman, NULL);
-		if (doc != NULL)
-		{
-			ed = IANJUTA_IS_EDITOR(doc) ? IANJUTA_EDITOR(doc) : NULL;
-		}
-	}
-	
-	return ed;
-}
-
 /* BreakItem functions
  *---------------------------------------------------------------------------*/
 
@@ -256,6 +237,8 @@ breakpoint_item_new (BreakpointsDBase *bd)
 	bi = g_new0 (BreakpointItem, 1);
 	bi->ref = 1;
 	bi->bd = bd;
+	bi->editor = NULL;
+	bi->handle = -1;
 	
 	bi->bp.type = 0;
 
@@ -374,7 +357,7 @@ breakpoints_dbase_set_in_editor (BreakpointsDBase *bd, BreakpointItem *bi)
 	{
 		line = bi->bp.line;
 	}
-
+	
 	/* Add new mark */
 	bi->handle = ianjuta_markable_mark (ed, line, bi->bp.enable ? BREAKPOINT_ENABLED : BREAKPOINT_DISABLED, NULL);
 }
@@ -400,6 +383,8 @@ breakpoints_dbase_clear_in_editor (BreakpointsDBase *bd, BreakpointItem *bi)
 		/* Remove old mark */
 		ianjuta_markable_unmark (ed, line, BREAKPOINT_ENABLED, NULL);
 		ianjuta_markable_unmark (ed, line, BREAKPOINT_DISABLED,NULL);
+		bi->bp.line = line;
+		bi->handle = -1;
 	}
 }
 
@@ -500,6 +485,137 @@ breakpoints_dbase_breakpoint_updated (BreakpointsDBase *bd, BreakpointItem *bi)
 	}
 }
 
+static void
+on_editor_saved (IAnjutaEditor *editor, const gchar* uri, BreakpointsDBase *bd)
+{
+	GtkTreeIter iter;
+	GtkTreeModel *model = GTK_TREE_MODEL (bd->model);
+	
+	g_return_if_fail (model != NULL);
+	/* Update breakpoint position */
+	if (gtk_tree_model_get_iter_first (model, &iter))
+	{
+		do
+		{
+			BreakpointItem *bi;
+		
+			gtk_tree_model_get (GTK_TREE_MODEL (bd->model), &iter, DATA_COLUMN, &bi, -1);
+
+			if ((bi->editor == editor) && (bi->handle != -1))
+			{
+				gint line;
+				
+				line = ianjuta_markable_location_from_handle (IANJUTA_MARKABLE (editor), bi->handle, NULL);
+				if (line != bi->bp.line)
+				{
+					bi->bp.line = line;
+					breakpoints_dbase_breakpoint_updated (bd, bi);
+				}
+			}	
+		} while (gtk_tree_model_iter_next (model, &iter));
+	}
+}
+
+static void
+breakpoints_dbase_connect_to_editor (BreakpointsDBase *bd, IAnjutaEditor* ed)
+{
+	if (!g_signal_handler_find (ed, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, bd))
+	{
+		/* signal is not already connected */
+		g_signal_connect (ed, "saved", G_CALLBACK (on_editor_saved), bd);
+	}
+}
+
+static void
+breakpoints_dbase_disconnect_from_editors (BreakpointsDBase *bd)
+{
+	GtkTreeIter iter;
+	GtkTreeModel *model = GTK_TREE_MODEL (bd->model);
+	
+	if (gtk_tree_model_get_iter_first (model, &iter))
+	{
+		do
+		{
+			BreakpointItem *bi;
+		
+			gtk_tree_model_get (model, &iter, DATA_COLUMN, &bi, -1);
+
+			if (bi->editor != NULL)
+			{
+				g_signal_handlers_disconnect_matched (bi->editor, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, bd);
+			}
+		} while (gtk_tree_model_iter_next (model, &iter));
+	}
+}
+
+static void
+breakpoints_dbase_set_all_in_editor (BreakpointsDBase* bd, IAnjutaEditor* te)
+{
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	gchar *uri;
+	
+	g_return_if_fail (te != NULL);
+	g_return_if_fail (bd != NULL);
+	g_return_if_fail (bd->treeview != NULL);
+	
+	uri = ianjuta_file_get_uri (IANJUTA_FILE (te), NULL);
+	if (uri == NULL)
+		return;
+	
+	if (!IANJUTA_IS_MARKABLE (te))
+	{
+		/* Nothing to do, editor does not support mark */
+		return;
+	}
+	
+	model = gtk_tree_view_get_model (bd->treeview);
+	
+	if (gtk_tree_model_get_iter_first (model, &iter))
+	{
+		do
+		{
+			BreakpointItem *bi;
+		
+			gtk_tree_model_get (model, &iter, DATA_COLUMN, &bi, -1);
+
+			if ((bi->editor == NULL) && (strcmp (uri, bi->uri) == 0))
+			{
+				bi->editor = te;
+				bi->handle = -1;
+				g_object_add_weak_pointer (G_OBJECT (te), (gpointer)&bi->editor);
+				breakpoints_dbase_connect_to_editor (bd, te);				
+			}
+			if (bi->editor == te)
+			{
+				breakpoints_dbase_set_in_editor (bd, bi);
+			}
+		} while (gtk_tree_model_iter_next (model, &iter));
+	}
+	g_free (uri);
+}
+
+static void
+on_added_current_editor (AnjutaPlugin *plugin, const char *name,
+							const GValue *value, gpointer user_data)
+{
+	BreakpointsDBase *bd = (BreakpointsDBase *)user_data;
+	GObject *editor;
+		
+	editor = g_value_get_object (value);
+	
+    /* Restore breakpoints */
+	if (IANJUTA_IS_EDITOR (editor))
+		breakpoints_dbase_set_all_in_editor (bd, IANJUTA_EDITOR (editor));
+}
+
+static void
+on_removed_current_editor (AnjutaPlugin *plugin,
+							  const char *name, gpointer data)
+{
+	/* Nothing do to here */
+}
+
 /* Private functions
  *---------------------------------------------------------------------------*/
 
@@ -619,6 +735,10 @@ breakpoints_dbase_add_in_debugger (BreakpointsDBase *bd, BreakpointItem *bi)
 	{
 		bi->changed = IANJUTA_DEBUGGER_BREAKPOINT_WITH_IGNORE;
 	}
+	if (bi->bp.enable != TRUE)
+	{
+		bi->changed = IANJUTA_DEBUGGER_BREAKPOINT_WITH_ENABLE;
+	}
 	
 	/* Add breakpoint in debugger */
 	if (bi->bp.type & IANJUTA_DEBUGGER_BREAKPOINT_ON_LINE)
@@ -641,7 +761,8 @@ breakpoints_dbase_add_in_debugger (BreakpointsDBase *bd, BreakpointItem *bi)
 					on_breakpoint_callback,
 					bi);
 	}
-	else if (bi->bp.type & IANJUTA_DEBUGGER_BREAKPOINT_ON_ADDRESS)
+	else if ((bi->bp.type & IANJUTA_DEBUGGER_BREAKPOINT_ON_ADDRESS)
+			 && dma_debugger_queue_is_supported(bd->debugger, HAS_ADDRESS_BREAKPOINT))
 	{
 		breakpoint_item_ref (bi);
 		ok = dma_queue_add_breakpoint_at_address (
@@ -673,7 +794,8 @@ breakpoints_dbase_update_in_debugger (BreakpointsDBase *bd, BreakpointItem *bi)
 		if (!ok) breakpoint_item_unref (bi);
 	}
 	
-	if (bi->changed & IANJUTA_DEBUGGER_BREAKPOINT_WITH_CONDITION)
+	if ((bi->changed & IANJUTA_DEBUGGER_BREAKPOINT_WITH_CONDITION)
+		&& dma_debugger_queue_is_supported(bd->debugger, HAS_CONDITION_BREAKPOINT))
 	{
 		bi->changed &= ~IANJUTA_DEBUGGER_BREAKPOINT_WITH_CONDITION;
 		breakpoint_item_ref (bi);
@@ -685,7 +807,8 @@ breakpoints_dbase_update_in_debugger (BreakpointsDBase *bd, BreakpointItem *bi)
 					bi);
 		if (!ok) breakpoint_item_unref (bi);
 	}
-	if (bi->changed & IANJUTA_DEBUGGER_BREAKPOINT_WITH_IGNORE)
+	if ((bi->changed & IANJUTA_DEBUGGER_BREAKPOINT_WITH_IGNORE)
+		&& dma_debugger_queue_is_supported(bd->debugger, HAS_IGNORE_BREAKPOINT))
 	{
 		bi->changed &= ~IANJUTA_DEBUGGER_BREAKPOINT_WITH_IGNORE;
 		breakpoint_item_ref (bi);
@@ -850,15 +973,20 @@ breakpoints_dbase_remove_all_in_debugger (BreakpointsDBase *bd)
 	
 	if (gtk_tree_model_get_iter_first (model, &iter))
 	{
+		gboolean next;
+		
 		do
 		{
 			BreakpointItem *bi;
 		
 			gtk_tree_model_get (model, &iter, DATA_COLUMN, &bi, -1);
-
+			next = gtk_tree_model_iter_next (model, &iter);
+				
 			bi->bp.id = 0;
 			breakpoints_dbase_breakpoint_updated (bd, bi);
-		} while (gtk_tree_model_iter_next (model, &iter));
+			/* breakpoints_dbase_breakpoint_updated can delete pending breakpoints
+			 * the iterator of the breakpoint can be invalidated */
+		} while (next);
 	}
 }
 
@@ -870,7 +998,7 @@ breakpoints_dbase_add_breakpoint (BreakpointsDBase *bd,  BreakpointItem *bi)
 	IAnjutaEditor* ed;
 	
 	/* Add in current editor if possible */
-	ed = get_current_editor (bd->plugin);
+	ed = dma_get_current_editor (ANJUTA_PLUGIN(bd->plugin));
 	if ((ed != NULL) && IANJUTA_IS_MARKABLE (ed))
 	{
 		gchar *uri;
@@ -881,6 +1009,7 @@ breakpoints_dbase_add_breakpoint (BreakpointsDBase *bd,  BreakpointItem *bi)
 			bi->editor = ed;
 			bi->handle = -1;
 			g_object_add_weak_pointer (G_OBJECT (ed), (gpointer)&bi->editor);
+			breakpoints_dbase_connect_to_editor (bd, ed);			
 		}
 		g_free (uri);
 	}
@@ -959,17 +1088,23 @@ breakpoints_dbase_remove_all (BreakpointsDBase *bd)
 {
 	GtkTreeIter iter;
 	GtkTreeModel *model = GTK_TREE_MODEL (bd->model);
+
 	
 	if (gtk_tree_model_get_iter_first (model, &iter))
 	{
+		gboolean next;
+		
 		do
 		{
 			BreakpointItem *bi;
 		
-			gtk_tree_model_get (GTK_TREE_MODEL (bd->model), &iter, DATA_COLUMN, &bi, -1);
-
+			gtk_tree_model_get (model, &iter, DATA_COLUMN, &bi, -1);
+			next = gtk_tree_model_iter_next (model, &iter);
+				
 			breakpoints_dbase_remove_breakpoint (bd, bi);
-		} while (gtk_tree_model_iter_next (model, &iter));
+			/* Avoid an infinite loop if the breakpoint is not removed due to
+			 * an error */
+		} while (next);
 	}
 }
 
@@ -989,21 +1124,42 @@ breakpoints_dbase_enable_all (BreakpointsDBase *bd, gboolean enable)
 		
 			gtk_tree_model_get (GTK_TREE_MODEL (bd->model), &iter, DATA_COLUMN, &bi, -1);
 
-			breakpoints_dbase_enable_breakpoint (bd, bi, TRUE);
+			breakpoints_dbase_enable_breakpoint (bd, bi, enable);
 		} while (gtk_tree_model_iter_next (model, &iter));
 	}
 }
 
+static BreakpointItem* 
+breakpoints_dbase_find_breakpoint_from_mark (BreakpointsDBase *bd, IAnjutaEditor *ed, guint line)
+{
+	GtkTreeIter iter;
+	GtkTreeModel *model = GTK_TREE_MODEL (bd->model);
+	
+	if (gtk_tree_model_get_iter_first (model, &iter))
+	{
+		do
+		{
+			BreakpointItem *bi;
+		
+			gtk_tree_model_get (model, &iter, DATA_COLUMN, &bi, -1);
+
+			if ((bi->editor == ed) && (bi->handle != -1)
+				&& (ianjuta_markable_location_from_handle (IANJUTA_MARKABLE(ed), bi->handle, NULL) == line))
+			{
+				return bi;
+			}
+		} while (gtk_tree_model_iter_next (model, &iter));
+	}
+	
+	return NULL;
+}
+
 static BreakpointItem* 
 breakpoints_dbase_find_breakpoint_from_line (BreakpointsDBase *bd, const gchar* uri, guint line)
 {
 	GtkTreeIter iter;
 	GtkTreeModel *model = GTK_TREE_MODEL (bd->model);
 	
-	g_return_val_if_fail (bd->treeview != NULL, NULL);
-	
-	model = gtk_tree_view_get_model (bd->treeview);
-	
 	if (gtk_tree_model_get_iter_first (model, &iter))
 	{
 		do
@@ -1034,7 +1190,8 @@ breakpoints_dbase_get_breakpoint_list (BreakpointsDBase *bd)
 		
 			gtk_tree_model_get (GTK_TREE_MODEL (bd->model), &iter, DATA_COLUMN, &bi, -1);
 			
-			if (bi->bp.temporary == FALSE)
+			if ((bi->bp.type & (IANJUTA_DEBUGGER_BREAKPOINT_ON_LINE | IANJUTA_DEBUGGER_BREAKPOINT_ON_FUNCTION))
+					&& (bi->bp.temporary == FALSE))
 			{
 				/* Only permanent breakpoint are saved */				
 				list = g_list_prepend (list, g_strdup_printf("%d:%s:%u:%u:%s", bi->bp.enable ? 1 : 0, bi->uri, bi->bp.line, bi->bp.ignore, bi->bp.condition == NULL ? "" : bi->bp.condition));
@@ -1095,16 +1252,33 @@ on_breakpoint_sharedlib_event (BreakpointsDBase *bd)
 	breakpoints_dbase_add_all_pending_in_debugger (bd);
 }
 
+static void
+on_program_running (BreakpointsDBase *bd)
+{
+	/* Deactivate breakpoint functions */
+	gtk_action_group_set_sensitive (bd->debugger_group, FALSE);
+}
+
 static void
 on_program_stopped (BreakpointsDBase *bd)
 {
 	g_return_if_fail (bd->debugger != NULL);
 
+	gtk_action_group_set_sensitive (bd->debugger_group, TRUE);
+	
 	/* Refresh breakpoint
 	 * Hit count could have changed by example */
 	breakpoints_dbase_list_all_in_debugger (bd);
 }
 
+static void
+on_program_exited (BreakpointsDBase *bd)
+{
+	g_return_if_fail (bd->debugger != NULL);
+
+	gtk_action_group_set_sensitive (bd->debugger_group, TRUE);
+}	
+
 static void
 on_program_unloaded (BreakpointsDBase *bd)
 {
@@ -1112,10 +1286,13 @@ on_program_unloaded (BreakpointsDBase *bd)
 	
 	breakpoints_dbase_remove_all_in_debugger (bd);
 	bd->debugger = NULL;
+	gtk_action_group_set_sensitive (bd->debugger_group, TRUE);
 
 	/* Disconnect from other debugger signal */
 	g_signal_handlers_disconnect_by_func (bd->plugin, G_CALLBACK (on_breakpoint_sharedlib_event), bd);
 	g_signal_handlers_disconnect_by_func (bd->plugin, G_CALLBACK (on_program_stopped), bd);
+	g_signal_handlers_disconnect_by_func (bd->plugin, G_CALLBACK (on_program_running), bd);
+	g_signal_handlers_disconnect_by_func (bd->plugin, G_CALLBACK (on_program_exited), bd);
 	g_signal_handlers_disconnect_by_func (bd->plugin, G_CALLBACK (on_program_unloaded), bd);
 }
 
@@ -1138,8 +1315,51 @@ on_program_loaded (BreakpointsDBase *bd)
 	g_signal_connect_swapped (bd->plugin, "sharedlib-event", G_CALLBACK (on_breakpoint_sharedlib_event), bd);
 	g_signal_connect_swapped (bd->plugin, "program-unloaded", G_CALLBACK (on_program_unloaded), bd);
 	g_signal_connect_swapped (bd->plugin, "program-stopped", G_CALLBACK (on_program_stopped), bd);
+	g_signal_connect_swapped (bd->plugin, "program-exited", G_CALLBACK (on_program_exited), bd);
+	g_signal_connect_swapped (bd->plugin, "program-running", G_CALLBACK (on_program_running), bd);
 }
 
+static void
+on_debugger_started (BreakpointsDBase *bd)
+{
+	GtkTreeViewColumn *column;	
+	DmaDebuggerQueue* debugger;
+	
+	debugger = dma_debug_manager_get_queue (ANJUTA_PLUGIN_DEBUG_MANAGER (bd->plugin));
+	
+	/* Remove breakpoint attributes not supported by current debugger */
+	if (!dma_debugger_queue_is_supported(debugger, HAS_ADDRESS_BREAKPOINT))
+	{
+		column = gtk_tree_view_get_column (bd->treeview, ADDRESS_COLUMN);
+		gtk_tree_view_column_set_visible (column, FALSE);
+	}
+	if (!dma_debugger_queue_is_supported(debugger, HAS_IGNORE_BREAKPOINT))
+	{
+		column = gtk_tree_view_get_column (bd->treeview, PASS_COLUMN);	
+		gtk_tree_view_column_set_visible (column, FALSE);
+	}
+	if (!dma_debugger_queue_is_supported(debugger, HAS_CONDITION_BREAKPOINT))
+	{
+		column = gtk_tree_view_get_column (bd->treeview, CONDITION_COLUMN);	
+		gtk_tree_view_column_set_visible (column, FALSE);
+	}
+}
+
+static void
+on_debugger_stopped (BreakpointsDBase *bd)
+{
+	/* Restore breakpoint attributes not supported by current debugger */
+	GtkTreeViewColumn *column;	
+	
+	column = gtk_tree_view_get_column (bd->treeview, ADDRESS_COLUMN);
+	gtk_tree_view_column_set_visible (column, TRUE);
+	column = gtk_tree_view_get_column (bd->treeview, PASS_COLUMN);	
+	gtk_tree_view_column_set_visible (column, TRUE);
+	column = gtk_tree_view_get_column (bd->treeview, CONDITION_COLUMN);	
+	gtk_tree_view_column_set_visible (column, TRUE);
+}
+
+
 /* Saving preferences callbacks
  *---------------------------------------------------------------------------*/
 
@@ -1179,8 +1399,8 @@ breakpoints_dbase_edit_breakpoint (BreakpointsDBase *bd, BreakpointItem *bi)
 	GladeXML *gxml;
 	GtkWidget *dialog;
 	GtkWidget *location_label, *location_entry;
-	GtkWidget *condition_entry;
-	GtkWidget *pass_entry;
+	GtkWidget *condition_entry, *condition_label;
+	GtkWidget *pass_entry, *pass_label;
 	gchar *buff;
 	gchar *location = NULL;
 	gchar *uri = NULL;
@@ -1194,8 +1414,20 @@ breakpoints_dbase_edit_breakpoint (BreakpointsDBase *bd, BreakpointItem *bi)
 	location_label = glade_xml_get_widget (gxml, "breakpoint_location_label");
 	location_entry = glade_xml_get_widget (gxml, "breakpoint_location_entry");
 	condition_entry = glade_xml_get_widget (gxml, "breakpoint_condition_entry");
+	condition_label = glade_xml_get_widget (gxml, "breakpoint_condition_label");
 	pass_entry = glade_xml_get_widget (gxml, "breakpoint_pass_entry");
+	pass_label = glade_xml_get_widget (gxml, "breakpoint_pass_label");
 
+	if (!dma_debugger_queue_is_supported(bd->debugger, HAS_IGNORE_BREAKPOINT))
+	{
+		gtk_widget_hide (pass_entry);
+		gtk_widget_hide (pass_label);
+	}
+	if (!dma_debugger_queue_is_supported(bd->debugger, HAS_CONDITION_BREAKPOINT))
+	{
+		gtk_widget_hide (condition_entry);
+		gtk_widget_hide (condition_label);
+	}
 	
 	if (bi == NULL)
 	{
@@ -1207,7 +1439,7 @@ breakpoints_dbase_edit_breakpoint (BreakpointsDBase *bd, BreakpointItem *bi)
 		gtk_widget_hide (location_label);
 
 		/* Get current editor and line */
-		te = get_current_editor (bd->plugin);
+		te = dma_get_current_editor (ANJUTA_PLUGIN(bd->plugin));
 		if (te != NULL)
 		{
 			uri = ianjuta_file_get_uri (IANJUTA_FILE (te), NULL);
@@ -1358,14 +1590,19 @@ on_toggle_breakpoint_activate (GtkAction * action, BreakpointsDBase *bd)
 	guint line;
 
 	/* Get current editor and line */
-	te = get_current_editor (bd->plugin);
+	te = dma_get_current_editor (ANJUTA_PLUGIN (bd->plugin));
 	if (te == NULL) return;       /* Missing editor */
 	uri = ianjuta_file_get_uri (IANJUTA_FILE (te), NULL);
 	if (uri == NULL) return;     /* File not saved yet, it's not possible to put a breakpoint in it */
 	line = ianjuta_editor_get_lineno (te, NULL);
 	
-	/* Find corresponding breakpoint */
-	bi = breakpoints_dbase_find_breakpoint_from_line (bd, uri, line);
+	/* Find corresponding breakpoint
+	 * Try to find right mark (it could have moved) first */
+	bi = breakpoints_dbase_find_breakpoint_from_mark (bd, te, line);
+	if (bi == NULL)
+	{
+		bi = breakpoints_dbase_find_breakpoint_from_line (bd, uri, line);
+	}
 
 	if (bi == NULL)
 	{
@@ -1488,7 +1725,7 @@ on_enable_breakpoint_activate (GtkAction * action, BreakpointsDBase *bd)
 	}
 }
 
-static GtkActionEntry actions_breakpoints[] = {
+static GtkActionEntry actions_debugger_breakpoints[] = {
 	{
 		"ActionMenuDmaBreakpoints",               /* Action name */
 		NULL,                                     /* Stock icon, if any */
@@ -1521,14 +1758,6 @@ static GtkActionEntry actions_breakpoints[] = {
 		N_("Remove a breakpoint"),                /* Tooltip */
 		G_CALLBACK (on_remove_breakpoint_activate)   /* action callback */
 	},
-	{
-		"ActionDmaJumpToBreakpoint",              /* Action name */
-		NULL,                                     /* Stock icon, if any */
-		N_("Jump to Breakpoint"),                 /* Display label */
-		NULL,                                     /* short-cut */
-		N_("Jump to breakpoint location"),        /* Tooltip */
-		G_CALLBACK (on_jump_to_breakpoint_activate)   /* action callback */
-	},
 	{
 		"ActionDmaEditBreakpoint",                /* Action name */
 		NULL,                                     /* Stock icon, if any */
@@ -1563,6 +1792,17 @@ static GtkActionEntry actions_breakpoints[] = {
 	},
 };		
 
+static GtkActionEntry actions_permanent_breakpoints[] = {
+	{
+		"ActionDmaJumpToBreakpoint",              /* Action name */
+		NULL,                                     /* Stock icon, if any */
+		N_("Jump to Breakpoint"),                 /* Display label */
+		NULL,                                     /* short-cut */
+		N_("Jump to breakpoint location"),        /* Tooltip */
+		G_CALLBACK (on_jump_to_breakpoint_activate)   /* action callback */
+	},
+};		
+
 /* Breakpoint list window
  *---------------------------------------------------------------------------*/
 
@@ -1573,10 +1813,43 @@ on_breakpoints_button_press (GtkWidget * widget, GdkEventButton * bevent, Breakp
 	{
 		AnjutaUI *ui;
 		GtkMenu *popup;
-		
+		GtkTreeModel *model;
+		GtkTreeSelection *selection;
+		GtkTreeIter iter;
+		gboolean valid;
+
 		ui = anjuta_shell_get_ui (ANJUTA_PLUGIN(bd->plugin)->shell, NULL);
 		popup =  GTK_MENU (gtk_ui_manager_get_widget (GTK_UI_MANAGER (ui), "/PopupBreakpoint"));
 
+		selection = gtk_tree_view_get_selection (bd->treeview);
+		valid = gtk_tree_selection_get_selected (selection, &model, &iter);
+		if (valid)
+		{
+			BreakpointItem *bi;
+			GtkAction *action;
+			const gchar* label;
+			const gchar* tooltip;
+	
+			gtk_tree_model_get (model, &iter, DATA_COLUMN, &bi, -1);
+			action = gtk_action_group_get_action (bd->debugger_group, "ActionDmaEnableDisableBreakpoint");
+			g_return_if_fail (action != NULL);
+			if (bi->bp.enable)
+			{
+				label =	N_("Disable Breakpoint");
+				tooltip = N_("Disable a breakpoint");
+			}
+			else
+			{
+				label =	N_("Enable Breakpoint");
+				tooltip = N_("Enable a breakpoint");
+			}
+				
+			g_object_set (G_OBJECT (action),
+					"label", label, 
+              				"tooltip", tooltip, 
+		              		NULL);				
+		}
+		
 		gtk_menu_popup (popup, NULL, NULL, NULL, NULL,
 						bevent->button, bevent->time);
 	}
@@ -1606,7 +1879,8 @@ create_breakpoint_gui(BreakpointsDBase *bd)
 
 	g_return_if_fail (bd->treeview == NULL);
 	g_return_if_fail (bd->window == NULL);
-	g_return_if_fail (bd->action_group == NULL);
+	g_return_if_fail (bd->debugger_group == NULL);
+	g_return_if_fail (bd->permanent_group == NULL);
 
 	/* breakpoints window */
 	bd->model = gtk_list_store_newv (COLUMNS_NB, column_type);
@@ -1640,11 +1914,17 @@ create_breakpoint_gui(BreakpointsDBase *bd)
 
 	/* Register menu actions */
 	ui = anjuta_shell_get_ui (ANJUTA_PLUGIN(bd->plugin)->shell, NULL);
-	bd->action_group =
+	bd->debugger_group =
 	anjuta_ui_add_action_group_entries (ui, "ActionGroupBreakpoint",
 										_("Breakpoint operations"),
-										actions_breakpoints,
-										G_N_ELEMENTS (actions_breakpoints),
+										actions_debugger_breakpoints,
+										G_N_ELEMENTS (actions_debugger_breakpoints),
+										GETTEXT_PACKAGE, TRUE, bd);
+	bd->permanent_group =
+	anjuta_ui_add_action_group_entries (ui, "ActionGroupBreakpoint",
+										_("Breakpoint operations"),
+										actions_permanent_breakpoints,
+										G_N_ELEMENTS (actions_permanent_breakpoints),
 										GETTEXT_PACKAGE, TRUE, bd);
 
 	/* Add breakpoint window */
@@ -1674,10 +1954,15 @@ destroy_breakpoint_gui (BreakpointsDBase *bd)
 
 	/* Remove menu actions */
 	ui = anjuta_shell_get_ui (ANJUTA_PLUGIN (bd->plugin)->shell, NULL);
-	if (bd->action_group)
+	if (bd->debugger_group)
 	{
-		anjuta_ui_remove_action_group (ui, bd->action_group);
-		bd->action_group = NULL;
+		anjuta_ui_remove_action_group (ui, bd->debugger_group);
+		bd->debugger_group = NULL;
+	}
+	if (bd->permanent_group)
+	{
+		anjuta_ui_remove_action_group (ui, bd->permanent_group);
+		bd->permanent_group = NULL;
 	}
 
 	/* Destroy breakpoint window */
@@ -1688,83 +1973,6 @@ destroy_breakpoint_gui (BreakpointsDBase *bd)
 	}
 }
 
-/* Public functions
- *---------------------------------------------------------------------------*/
-
-void
-breakpoints_dbase_set_all_in_editor (BreakpointsDBase* bd, IAnjutaEditor* te)
-{
-	GtkTreeModel *model;
-	GtkTreeIter iter;
-	gchar *uri;
-	
-	g_return_if_fail (te != NULL);
-	g_return_if_fail (bd != NULL);
-	g_return_if_fail (bd->treeview != NULL);
-	
-	uri = ianjuta_file_get_uri (IANJUTA_FILE (te), NULL);
-	if (uri == NULL)
-		return;
-	
-	if (!IANJUTA_IS_MARKABLE (te))
-	{
-		/* Nothing to do, editor does not support mark */
-		return;
-	}
-	ianjuta_markable_delete_all_markers (IANJUTA_MARKABLE(te), BREAKPOINT_ENABLED, NULL);
-	ianjuta_markable_delete_all_markers (IANJUTA_MARKABLE(te), BREAKPOINT_DISABLED, NULL);
-	
-	model = gtk_tree_view_get_model (bd->treeview);
-	
-	if (gtk_tree_model_get_iter_first (model, &iter))
-	{
-		do
-		{
-			BreakpointItem *bi;
-		
-			gtk_tree_model_get (model, &iter, DATA_COLUMN, &bi, -1);
-
-			if ((bi->editor == NULL) && (strcmp (uri, bi->uri) == 0))
-			{
-				bi->editor = te;
-				bi->handle = -1;
-				g_object_add_weak_pointer (G_OBJECT (te), (gpointer)&bi->editor);
-			}
-			breakpoints_dbase_set_in_editor (bd, bi);
-		} while (gtk_tree_model_iter_next (model, &iter));
-	}
-	g_free (uri);
-}
-
-void
-breakpoints_dbase_clear_all_in_editor (BreakpointsDBase* bd, IAnjutaEditor* te)
-{
-	GtkTreeModel *model;
-	GtkTreeIter iter;
-
-	g_return_if_fail (te != NULL);
-	g_return_if_fail (bd != NULL);
-	g_return_if_fail (bd->treeview != NULL);
-
-	model = gtk_tree_view_get_model (bd->treeview);
-	
-	if (gtk_tree_model_get_iter_first (model, &iter))
-	{
-		do
-		{
-			BreakpointItem *bi;
-		
-			gtk_tree_model_get (model, &iter, DATA_COLUMN, &bi, -1);
-
-			if (bi->editor == te)
-			{
-				g_object_remove_weak_pointer (G_OBJECT (te), (gpointer *)(gpointer)&bi->editor);
-				bi->editor = NULL;	
-			}
-		} while (gtk_tree_model_iter_next (model, &iter));
-	}
-}
-
 /* Constructor & Destructor
  *---------------------------------------------------------------------------*/
 
@@ -1786,9 +1994,17 @@ breakpoints_dbase_new (DebugManagerPlugin *plugin)
 	g_signal_connect (ANJUTA_PLUGIN(bd->plugin)->shell, "load-session",
 				  G_CALLBACK (on_session_load), bd);
 		
-	/* Connect on stop debugger and load program */
+	/* Connect on load program */
 	g_signal_connect_swapped (bd->plugin, "program-loaded", G_CALLBACK (on_program_loaded), bd);
+	g_signal_connect_swapped (bd->plugin, "debugger-started", G_CALLBACK (on_debugger_started), bd);
+	g_signal_connect_swapped (bd->plugin, "debugger-stopped", G_CALLBACK (on_debugger_stopped), bd);
 
+	bd->editor_watch = 
+		anjuta_plugin_add_watch (ANJUTA_PLUGIN(bd->plugin), "document_manager_current_editor",
+								 on_added_current_editor,
+								 on_removed_current_editor, bd);
+						 
+	
 	return bd;
 }
 
@@ -1800,7 +2016,9 @@ breakpoints_dbase_destroy (BreakpointsDBase * bd)
 	/* Disconnect all signal */
 	g_signal_handlers_disconnect_matched (ANJUTA_PLUGIN(bd->plugin)->shell, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, bd);
 	g_signal_handlers_disconnect_matched (bd->plugin, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, bd);
-
+	breakpoints_dbase_disconnect_from_editors (bd);
+	anjuta_plugin_remove_watch (ANJUTA_PLUGIN(bd->plugin), bd->editor_watch, FALSE);
+	
 	/* This is necessary to clear the editor of breakpoint markers */
 	breakpoints_dbase_remove_all (bd);
 	
