@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 /*
@@ -56,11 +56,11 @@
 #define SPARSE_VIEW_PROGRAM_COUNTER         3
 #define SPARSE_VIEW_LINEMARKER              4
 
-#define MARKER_PIXMAP_BOOKMARK "bookmark.png"
-#define MARKER_PIXMAP_LINEMARKER "linemarker.png"
-#define MARKER_PIXMAP_PROGRAM_COUNTER "program-counter.png"
-#define MARKER_PIXMAP_BREAKPOINT_DISABLED "breakpoint-disabled.png"
-#define MARKER_PIXMAP_BREAKPOINT_ENABLED "breakpoint-enabled.png"
+#define MARKER_PIXMAP_BOOKMARK "anjuta-bookmark-16.png"
+#define MARKER_PIXMAP_LINEMARKER "anjuta-linemark-16.png"
+#define MARKER_PIXMAP_PROGRAM_COUNTER "anjuta-pcmark-16.png"
+#define MARKER_PIXMAP_BREAKPOINT_DISABLED "anjuta-breakpoint-disabled-16.png"
+#define MARKER_PIXMAP_BREAKPOINT_ENABLED "anjuta-breakpoint-enabled-16.png"
 
 /* Types
  *---------------------------------------------------------------------------*/
@@ -944,6 +944,26 @@ dma_sparse_view_goto (DmaSparseView *view, guint location)
 	gtk_adjustment_value_changed (view->priv->vadjustment);
 }
 
+guint
+dma_sparse_view_get_location (DmaSparseView *view)
+{
+	GtkTextMark *mark;
+	GtkTextBuffer *buffer;
+	GtkTextIter iter;
+	DmaSparseIter buf_iter;
+	gint line;
+	
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(view));
+	mark = gtk_text_buffer_get_insert (buffer);
+	gtk_text_buffer_get_iter_at_mark (buffer, &iter, mark);
+	line = gtk_text_iter_get_line (&iter);
+		
+	dma_sparse_iter_copy (&buf_iter, &view->priv->start);
+	dma_sparse_iter_forward_lines (&buf_iter, line);
+	
+	return dma_sparse_iter_get_address (&buf_iter);
+}
+
 /* GtkWidget functions
  *---------------------------------------------------------------------------*/
 
