@@ -34,6 +34,7 @@
 #include <libanjuta/anjuta-debug.h>
 #include <libanjuta/interfaces/ianjuta-editor.h>
 #include <libanjuta/interfaces/ianjuta-document-manager.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 
 #include "utilities.h"
 #include "info.h"
@@ -193,7 +194,7 @@ dma_thread_clear_all_stack_trace (StackTrace *self)
 }
 
 static void
-on_stack_trace_updated (const GList *stack, gpointer user_data)
+on_stack_trace_updated (const GList *stack, gpointer user_data, GError *error)
 {
 	StackTrace *self = (StackTrace *)user_data;
 	const GList *node;
@@ -202,6 +203,8 @@ on_stack_trace_updated (const GList *stack, gpointer user_data)
 	gboolean exist;
 	GdkPixbuf *pic;
 
+	if (error != NULL) return;
+	
 	model = GTK_LIST_STORE (self->current->model);
 
 	pic = gdk_pixbuf_new_from_file (ANJUTA_PIXMAP_POINTER, NULL);
@@ -287,7 +290,7 @@ on_stack_trace_updated (const GList *stack, gpointer user_data)
 		adr = g_strdup_printf ("0x%x", frame->address);
 		if (frame->file)
 		{
-			uri = g_strconcat ("file://", frame->file, NULL);
+			uri = gnome_vfs_get_uri_from_local_path(frame->file);
 			file = strrchr(uri, '/') + 1;
 		}
 		else
@@ -639,16 +642,20 @@ create_stack_trace_gui(StackTrace *st)
 	gtk_tree_view_column_set_title (column, _("Function"));
 	gtk_tree_view_append_column (st->treeview, column);
 	
-	column = gtk_tree_view_column_new ();
-	renderer = gtk_cell_renderer_text_new ();
-	gtk_tree_view_column_pack_start (column, renderer, TRUE);
-	gtk_tree_view_column_add_attribute (column, renderer, "text",
-										STACK_TRACE_ADDR_COLUMN);
-	gtk_tree_view_column_add_attribute (column, renderer, "foreground",
-										STACK_TRACE_COLOR_COLUMN);
-	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-	gtk_tree_view_column_set_title (column, _("Address"));
-	gtk_tree_view_append_column (st->treeview, column);
+	if (dma_debugger_queue_is_supported (st->debugger, HAS_CPU))
+	{
+		/* Display address only if debugger has such concept */
+		column = gtk_tree_view_column_new ();
+		renderer = gtk_cell_renderer_text_new ();
+		gtk_tree_view_column_pack_start (column, renderer, TRUE);
+		gtk_tree_view_column_add_attribute (column, renderer, "text",
+											STACK_TRACE_ADDR_COLUMN);
+		gtk_tree_view_column_add_attribute (column, renderer, "foreground",
+											STACK_TRACE_COLOR_COLUMN);
+		gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+		gtk_tree_view_column_set_title (column, _("Address"));
+		gtk_tree_view_append_column (st->treeview, column);
+	}
 	
 	column = gtk_tree_view_column_new ();
 	renderer = gtk_cell_renderer_text_new ();
@@ -706,17 +713,25 @@ on_program_moved (StackTrace *self, guint pid, gint thread)
 }
 
 static void
-on_debugger_started (StackTrace *self)
+on_program_exited (StackTrace *self)
 {
-	self->current_update = 0;
-	create_stack_trace_gui (self);
+	g_signal_handlers_disconnect_by_func (self->plugin, G_CALLBACK (on_program_exited), self);
+	g_signal_handlers_disconnect_by_func (self->plugin, G_CALLBACK (on_program_moved), self);
+	g_signal_handlers_disconnect_by_func (self->plugin, G_CALLBACK (on_frame_changed), self);
+	
+	dma_thread_clear_all_stack_trace (self);
+	destroy_stack_trace_gui (self);
 }
 
 static void
-on_debugger_stopped (StackTrace *self)
+on_program_started (StackTrace *self)
 {
-	dma_thread_clear_all_stack_trace (self);
-	destroy_stack_trace_gui (self);
+	self->current_update = 0;
+	create_stack_trace_gui (self);
+	
+	g_signal_connect_swapped (self->plugin, "program-exited", G_CALLBACK (on_program_exited), self);
+	g_signal_connect_swapped (self->plugin, "program-moved", G_CALLBACK (on_program_moved), self);
+	g_signal_connect_swapped (self->plugin, "frame-changed", G_CALLBACK (on_frame_changed), self);
 }
 
 /* Public functions
@@ -746,10 +761,7 @@ stack_trace_new (DebugManagerPlugin *plugin)
 											G_N_ELEMENTS (actions_stack_trace),
 											GETTEXT_PACKAGE, TRUE, st);
 
-	g_signal_connect_swapped (st->plugin, "debugger-started", G_CALLBACK (on_debugger_started), st);
-	g_signal_connect_swapped (st->plugin, "debugger-stopped", G_CALLBACK (on_debugger_stopped), st);
-	g_signal_connect_swapped (st->plugin, "program-moved", G_CALLBACK (on_program_moved), st);
-	g_signal_connect_swapped (st->plugin, "frame-changed", G_CALLBACK (on_frame_changed), st);
+	g_signal_connect_swapped (st->plugin, "program-started", G_CALLBACK (on_program_started), st);
 	
 	return st;
 }
@@ -762,13 +774,7 @@ stack_trace_free (StackTrace * st)
 	g_return_if_fail (st != NULL);
 
 	/* Disconnect from debugger */
-	if (st->debugger != NULL)
-	{	
-		g_signal_handlers_disconnect_by_func (st->plugin, G_CALLBACK (on_debugger_started), st);
-		g_signal_handlers_disconnect_by_func (st->plugin, G_CALLBACK (on_debugger_stopped), st);
-		g_signal_handlers_disconnect_by_func (st->plugin, G_CALLBACK (on_program_moved), st);
-		g_signal_handlers_disconnect_by_func (st->plugin, G_CALLBACK (on_frame_changed), st);
-	}
+	g_signal_handlers_disconnect_matched (st->plugin, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, st);	
 
 	/* Remove menu actions */
 	ui = anjuta_shell_get_ui (ANJUTA_PLUGIN (st->plugin)->shell, NULL);
