@@ -32,10 +32,12 @@
 #include "debugger.h"
 
 #include <libanjuta/interfaces/ianjuta-debugger.h>
-#include <libanjuta/interfaces/ianjuta-breakpoint-debugger.h>
+#include <libanjuta/interfaces/ianjuta-debugger-breakpoint.h>
 #include <libanjuta/interfaces/ianjuta-cpu-debugger.h>
-#include <libanjuta/interfaces/ianjuta-variable-debugger.h>
+#include <libanjuta/interfaces/ianjuta-debugger-variable.h>
+#include <libanjuta/interfaces/ianjuta-terminal.h>
 #include <libanjuta/anjuta-plugin.h>
+#include <signal.h>
 
 /* Plugin type
  *---------------------------------------------------------------------------*/
@@ -53,6 +55,9 @@ struct _GdbPlugin
 
 	/* Log message window */
 	IAnjutaMessageView *view;
+
+	/* Terminal */
+	pid_t term_pid;
 };
 
 struct _GdbPluginClass
@@ -60,6 +65,95 @@ struct _GdbPluginClass
 	AnjutaPluginClass parent_class;
 };
 
+/* Terminal functions
+ *---------------------------------------------------------------------------*/
+
+static void
+gdb_plugin_stop_terminal (GdbPlugin* plugin)
+{
+	DEBUG_PRINT ("In function: gdb_plugin_stop_terminal()");
+
+	if (plugin->term_pid > 0) {
+		kill (plugin->term_pid, SIGTERM);
+		plugin->term_pid = -1;
+	}
+}
+
+static gchar*
+gdb_plugin_start_terminal (GdbPlugin* plugin)
+{
+	gchar *file, *cmd;
+       	gchar *tty = NULL;
+	IAnjutaTerminal *term;
+
+	DEBUG_PRINT ("In function: gdb_plugin_start_terminal() previous pid %d", plugin->term_pid);
+
+	/* Close previous terminal if needed */
+	gdb_plugin_stop_terminal (plugin);
+
+	/* Get terminal plugin */	
+	term = anjuta_shell_get_interface (ANJUTA_PLUGIN (plugin)->shell, IAnjutaTerminal, NULL);
+	if (term == NULL)
+	{
+
+		anjuta_util_dialog_error (GTK_WINDOW (ANJUTA_PLUGIN (plugin)->shell),
+					  _("Anjuta terminal plugin is not installed. The program will be run without a terminal."));
+
+		return NULL;
+	}
+
+	/* Check if anjuta launcher is here */
+	if (anjuta_util_prog_is_installed ("anjuta_launcher", TRUE) == FALSE)
+	{
+		return NULL;
+	}
+
+	file = anjuta_util_get_a_tmp_file();
+	if (mkfifo (file, 0664) < 0)
+	{
+		anjuta_util_dialog_error (GTK_WINDOW (ANJUTA_PLUGIN (plugin)->shell),
+					  _("Fail to create fifo file named %s. The program will be run without a terminal"), file);
+		g_free (file);
+
+		return NULL;
+	}
+
+	/* Launch terminal */
+	cmd = g_strconcat ("anjuta_launcher --__debug_terminal ", file, NULL);
+	plugin->term_pid = ianjuta_terminal_execute_command (term, NULL, cmd, NULL);
+	g_free (cmd);
+
+	if (plugin->term_pid > 0)
+	{
+		/*
+		 * Warning: call to fopen() may be blocked if the terminal is
+		 * not properly started. I don't know how to handle this. May
+		 * be opening as non-blocking will solve this .
+		 */
+		g_file_get_contents (file, &tty, NULL, NULL);  /* Ok, take the risk. */
+		if (tty)
+		{       
+			g_strchomp (tty);	/* anjuta_launcher add a end of line after the terminal name */
+			if (strcmp(tty, "__ERROR__") == 0)
+			{
+				g_free (tty);
+				tty = NULL;
+			}
+		}
+	}
+	remove (file);
+	g_free (file);
+
+	if (tty == NULL)
+	{
+		anjuta_util_dialog_error (GTK_WINDOW (ANJUTA_PLUGIN (plugin)->shell),
+					  _("Cannot start terminal for debugging."));
+		gdb_plugin_stop_terminal (plugin);
+	}
+
+	return tty;
+}
+
 /* Private functions
  *---------------------------------------------------------------------------*/
 
@@ -71,6 +165,7 @@ on_debugger_stopped (GdbPlugin *self, GError *err)
 		g_signal_handlers_disconnect_by_func (self, G_CALLBACK (on_debugger_stopped), self);	
 		debugger_free (self->debugger);
 		self->debugger = NULL;
+		gdb_plugin_stop_terminal (self);
 	}
 }
 
@@ -138,6 +233,7 @@ gdb_plugin_instance_init (GObject* obj)
 
 	this->debugger = NULL;
 	this->output_callback = NULL;
+	this->term_pid = 0;
 }
 
 /* dispose is the first destruction step. It is used to unref object created
@@ -204,14 +300,19 @@ idebugger_get_state (IAnjutaDebugger *plugin, GError **err)
 
 static gboolean
 idebugger_load (IAnjutaDebugger *plugin, const gchar *file, const gchar* mime_type,
-				const GList *search_dirs, gboolean terminal, GError **err)
+				const GList *search_dirs, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 	gboolean is_libtool = FALSE;
 
 	/* Check allowed mime type */
-	if (strcmp (mime_type, "application/x-executable") == 0)
+	if (mime_type == NULL)
 	{
+		/* Hope that the target is supported */
+	}
+	else if (strcmp (mime_type, "application/x-executable") == 0)
+	{
+		/* Supported target */
 	}
 	else if (strcmp (mime_type, "application/x-shellscript") == 0)
 	{
@@ -222,20 +323,18 @@ idebugger_load (IAnjutaDebugger *plugin, const gchar *file, const gchar* mime_ty
 	}
 	else if (strcmp (mime_type, "application/x-core") == 0)
 	{
+		/* Supported target */
 	}
 	else
 	{
-		//return IANJUTA_DEBUGGER_NOT_IMPLEMENTED;
+		/* Not supported target */
 		return TRUE;
 	}
 	
-	// a NULL filename could be used to check supported mime type
-	if (file == NULL) return TRUE;
-		
 	// Start debugger
 	gdb_plugin_initialize (this);
 	
-	return debugger_start (this->debugger, search_dirs, file, is_libtool, terminal);
+	return debugger_start (this->debugger, search_dirs, file, is_libtool);
 }
 
 static gboolean
@@ -255,18 +354,21 @@ idebugger_attach (IAnjutaDebugger *plugin, pid_t pid, const GList *search_dirs,
 
 	// Start debugger
 	gdb_plugin_initialize (this);
-	debugger_start (this->debugger, search_dirs, NULL, FALSE, FALSE);
+	debugger_start (this->debugger, search_dirs, NULL, FALSE);
 	debugger_attach_process (this->debugger, pid);
 	
 	return TRUE;
 }
 
 static gboolean
-idebugger_start (IAnjutaDebugger *plugin, const gchar *argument, GError **err)
+idebugger_start (IAnjutaDebugger *plugin, const gchar *argument, gboolean terminal, gboolean stop, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
+	gchar *tty;
 
-	debugger_start_program (this->debugger, argument);
+	tty = terminal ? gdb_plugin_start_terminal (this) : NULL;
+	debugger_start_program (this->debugger, argument, tty, stop);
+	g_free (tty);
 
 	return TRUE;
 }
@@ -672,11 +774,22 @@ idebugger_iface_init (IAnjutaDebuggerIface *iface)
 }
 
 
-/* Implementation of IAnjutaBreakpointDebugger interface
+/* Implementation of IAnjutaDebuggerBreakpoint interface
  *---------------------------------------------------------------------------*/
 
+static gint
+idebugger_breakpoint_implement (IAnjutaDebuggerBreakpoint *plugin, GError **err)
+{
+	/* gdb implement all interface methods */
+	return IANJUTA_DEBUGGER_BREAKPOINT_SET_AT_ADDRESS
+		| IANJUTA_DEBUGGER_BREAKPOINT_SET_AT_FUNCTION
+		| IANJUTA_DEBUGGER_BREAKPOINT_ENABLE
+		| IANJUTA_DEBUGGER_BREAKPOINT_IGNORE
+		| IANJUTA_DEBUGGER_BREAKPOINT_CONDITION;
+}
+
 static gboolean
-ibreakpoint_debugger_add_breakpoint_at_line (IAnjutaBreakpointDebugger *plugin, const gchar* file, guint line, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+idebugger_breakpoint_add_at_line (IAnjutaDebuggerBreakpoint *plugin, const gchar* file, guint line, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 	
@@ -686,7 +799,7 @@ ibreakpoint_debugger_add_breakpoint_at_line (IAnjutaBreakpointDebugger *plugin,
 }
 
 static gboolean
-ibreakpoint_debugger_add_breakpoint_at_function (IAnjutaBreakpointDebugger *plugin, const gchar* file, const gchar* function, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+idebugger_breakpoint_add_at_function (IAnjutaDebuggerBreakpoint *plugin, const gchar* file, const gchar* function, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 	
@@ -696,7 +809,7 @@ ibreakpoint_debugger_add_breakpoint_at_function (IAnjutaBreakpointDebugger *plug
 }
 
 static gboolean
-ibreakpoint_debugger_add_breakpoint_at_address (IAnjutaBreakpointDebugger *plugin, guint address, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+idebugger_breakpoint_add_at_address (IAnjutaDebuggerBreakpoint *plugin, guint address, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 	
@@ -706,7 +819,7 @@ ibreakpoint_debugger_add_breakpoint_at_address (IAnjutaBreakpointDebugger *plugi
 }
 
 static gboolean
-ibreakpoint_debugger_enable_breakpoint (IAnjutaBreakpointDebugger *plugin, guint id, gboolean enable, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+idebugger_breakpoint_enable (IAnjutaDebuggerBreakpoint *plugin, guint id, gboolean enable, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 	
@@ -716,7 +829,7 @@ ibreakpoint_debugger_enable_breakpoint (IAnjutaBreakpointDebugger *plugin, guint
 }
 
 static gboolean
-ibreakpoint_debugger_ignore_breakpoint (IAnjutaBreakpointDebugger *plugin, guint id, guint ignore, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+idebugger_breakpoint_ignore (IAnjutaDebuggerBreakpoint *plugin, guint id, guint ignore, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 	
@@ -726,7 +839,7 @@ ibreakpoint_debugger_ignore_breakpoint (IAnjutaBreakpointDebugger *plugin, guint
 }
 
 static gboolean
-ibreakpoint_debugger_condition_breakpoint (IAnjutaBreakpointDebugger *plugin, guint id, const gchar *condition, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+idebugger_breakpoint_condition (IAnjutaDebuggerBreakpoint *plugin, guint id, const gchar *condition, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 	
@@ -736,7 +849,7 @@ ibreakpoint_debugger_condition_breakpoint (IAnjutaBreakpointDebugger *plugin, gu
 }
 
 static gboolean
-ibreakpoint_debugger_remove_breakpoint (IAnjutaBreakpointDebugger *plugin, guint id, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+idebugger_breakpoint_remove (IAnjutaDebuggerBreakpoint *plugin, guint id, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -745,16 +858,28 @@ ibreakpoint_debugger_remove_breakpoint (IAnjutaBreakpointDebugger *plugin, guint
 	return TRUE;
 }
 
-static void
-ibreakpoint_debugger_iface_init (IAnjutaBreakpointDebuggerIface *iface)
+static gboolean
+idebugger_breakpoint_list (IAnjutaDebuggerBreakpoint *plugin, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
 {
-	iface->set_breakpoint_at_line = ibreakpoint_debugger_add_breakpoint_at_line;
-	iface->clear_breakpoint = ibreakpoint_debugger_remove_breakpoint;
-	iface->set_breakpoint_at_address = ibreakpoint_debugger_add_breakpoint_at_address;
-	iface->set_breakpoint_at_function = ibreakpoint_debugger_add_breakpoint_at_function;
-	iface->enable_breakpoint = ibreakpoint_debugger_enable_breakpoint;
-	iface->ignore_breakpoint = ibreakpoint_debugger_ignore_breakpoint;
-	iface->condition_breakpoint = ibreakpoint_debugger_condition_breakpoint;
+	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
+
+	debugger_list_breakpoint (this->debugger, callback, user_data);
+
+	return TRUE;
+}
+
+static void
+idebugger_breakpoint_iface_init (IAnjutaDebuggerBreakpointIface *iface)
+{
+	iface->implement = idebugger_breakpoint_implement;
+	iface->set_at_line = idebugger_breakpoint_add_at_line;
+	iface->clear = idebugger_breakpoint_remove;
+	iface->list = idebugger_breakpoint_list;
+	iface->set_at_address = idebugger_breakpoint_add_at_address;
+	iface->set_at_function = idebugger_breakpoint_add_at_function;
+	iface->enable = idebugger_breakpoint_enable;
+	iface->ignore = idebugger_breakpoint_ignore;
+	iface->condition = idebugger_breakpoint_condition;
 }
 
 /* Implementation of IAnjutaCpuDebugger interface
@@ -810,6 +935,36 @@ icpu_debugger_disassemble (IAnjutaCpuDebugger *plugin, guint address, guint leng
 	return TRUE;
 }
 
+static gboolean
+icpu_debugger_stepi_in (IAnjutaCpuDebugger *plugin, GError **err)
+{
+	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
+	
+	debugger_stepi_in (this->debugger);
+
+	return TRUE;
+}
+
+static gboolean
+icpu_debugger_stepi_over (IAnjutaCpuDebugger *plugin, GError **err)
+{
+	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
+	
+	debugger_stepi_over (this->debugger);
+
+	return TRUE;
+}
+
+static gboolean
+icpu_debugger_run_to_address (IAnjutaCpuDebugger *plugin, guint address, GError **err)
+{
+	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
+	
+	debugger_run_to_address (this->debugger, address);
+
+	return TRUE;
+}
+
 static void
 icpu_debugger_iface_init (IAnjutaCpuDebuggerIface *iface)
 {
@@ -818,13 +973,16 @@ icpu_debugger_iface_init (IAnjutaCpuDebuggerIface *iface)
 	iface->write_register = icpu_debugger_write_register;
 	iface->inspect_memory = icpu_debugger_inspect_memory;
 	iface->disassemble = icpu_debugger_disassemble;
+	iface->stepi_in = icpu_debugger_stepi_in;
+	iface->stepi_over = icpu_debugger_stepi_over;
+	iface->run_to_address = icpu_debugger_run_to_address;
 }
 
-/* Implementation of IAnjutaVariableDebugger interface
+/* Implementation of IAnjutaDebuggerVariable interface
  *---------------------------------------------------------------------------*/
 
 static gboolean
-ivariable_debugger_delete (IAnjutaVariableDebugger *plugin, const gchar *name, GError **error)
+idebugger_variable_destroy (IAnjutaDebuggerVariable *plugin, const gchar *name, GError **error)
 {
 	GdbPlugin *gdb = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -834,7 +992,7 @@ ivariable_debugger_delete (IAnjutaVariableDebugger *plugin, const gchar *name, G
 }
 
 static gboolean
-ivariable_debugger_evaluate (IAnjutaVariableDebugger *plugin, const gchar *name, IAnjutaDebuggerCallback callback , gpointer user_data, GError **error)
+idebugger_variable_evaluate (IAnjutaDebuggerVariable *plugin, const gchar *name, IAnjutaDebuggerCallback callback , gpointer user_data, GError **error)
 {
 	GdbPlugin *gdb = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -844,7 +1002,7 @@ ivariable_debugger_evaluate (IAnjutaVariableDebugger *plugin, const gchar *name,
 }
 
 static gboolean
-ivariable_debugger_assign (IAnjutaVariableDebugger *plugin, const gchar *name, const gchar *value, GError **error)
+idebugger_variable_assign (IAnjutaDebuggerVariable *plugin, const gchar *name, const gchar *value, GError **error)
 {
 	GdbPlugin *gdb = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -854,7 +1012,7 @@ ivariable_debugger_assign (IAnjutaVariableDebugger *plugin, const gchar *name, c
 }
 
 static gboolean
-ivariable_debugger_list_children (IAnjutaVariableDebugger *plugin, const gchar *name, IAnjutaDebuggerCallback callback , gpointer user_data, GError **error)
+idebugger_variable_list_children (IAnjutaDebuggerVariable *plugin, const gchar *name, IAnjutaDebuggerCallback callback , gpointer user_data, GError **error)
 {
 	GdbPlugin *gdb = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -864,7 +1022,7 @@ ivariable_debugger_list_children (IAnjutaVariableDebugger *plugin, const gchar *
 }
 
 static gboolean
-ivariable_debugger_create (IAnjutaVariableDebugger *plugin, const gchar *name, IAnjutaDebuggerCallback callback , gpointer user_data, GError **error)
+idebugger_variable_create (IAnjutaDebuggerVariable *plugin, const gchar *name, IAnjutaDebuggerCallback callback , gpointer user_data, GError **error)
 {
 	GdbPlugin *gdb = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -874,7 +1032,7 @@ ivariable_debugger_create (IAnjutaVariableDebugger *plugin, const gchar *name, I
 }
 
 static gboolean
-ivariable_debugger_update (IAnjutaVariableDebugger *plugin, IAnjutaDebuggerCallback callback , gpointer user_data, GError **error)
+idebugger_variable_update (IAnjutaDebuggerVariable *plugin, IAnjutaDebuggerCallback callback , gpointer user_data, GError **error)
 {
 	GdbPlugin *gdb = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -884,21 +1042,21 @@ ivariable_debugger_update (IAnjutaVariableDebugger *plugin, IAnjutaDebuggerCallb
 }
 
 static void
-ivariable_debugger_iface_init (IAnjutaVariableDebuggerIface *iface)
+idebugger_variable_iface_init (IAnjutaDebuggerVariableIface *iface)
 {
-	iface->delete_var = ivariable_debugger_delete;
-	iface->evaluate = ivariable_debugger_evaluate;
-	iface->assign = ivariable_debugger_assign;
-	iface->list_children = ivariable_debugger_list_children;
-	iface->create = ivariable_debugger_create;
-	iface->update = ivariable_debugger_update;
+	iface->destroy = idebugger_variable_destroy;
+	iface->evaluate = idebugger_variable_evaluate;
+	iface->assign = idebugger_variable_assign;
+	iface->list_children = idebugger_variable_list_children;
+	iface->create = idebugger_variable_create;
+	iface->update = idebugger_variable_update;
 }
 
 ANJUTA_PLUGIN_BEGIN (GdbPlugin, gdb_plugin);
 ANJUTA_PLUGIN_ADD_INTERFACE(idebugger, IANJUTA_TYPE_DEBUGGER);
-ANJUTA_PLUGIN_ADD_INTERFACE(ibreakpoint_debugger, IANJUTA_TYPE_BREAKPOINT_DEBUGGER);
+ANJUTA_PLUGIN_ADD_INTERFACE(idebugger_breakpoint, IANJUTA_TYPE_DEBUGGER_BREAKPOINT);
 ANJUTA_PLUGIN_ADD_INTERFACE(icpu_debugger, IANJUTA_TYPE_CPU_DEBUGGER);
-ANJUTA_PLUGIN_ADD_INTERFACE(ivariable_debugger, IANJUTA_TYPE_VARIABLE_DEBUGGER);
+ANJUTA_PLUGIN_ADD_INTERFACE(idebugger_variable, IANJUTA_TYPE_DEBUGGER_VARIABLE);
 ANJUTA_PLUGIN_END;
 
 ANJUTA_SIMPLE_PLUGIN (GdbPlugin, gdb_plugin);
