@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include <config.h>
@@ -24,7 +24,6 @@
 #include <libanjuta/anjuta-debug.h>
 #include <libanjuta/interfaces/ianjuta-help.h>
 #include <libanjuta/interfaces/ianjuta-document-manager.h>
-#include <libanjuta/interfaces/ianjuta-message-manager.h>
 #include <libanjuta/interfaces/ianjuta-project-manager.h>
 #include <libanjuta/interfaces/ianjuta-file-manager.h>
 #include <libanjuta/interfaces/ianjuta-file.h>
@@ -32,12 +31,13 @@
 #include <libanjuta/interfaces/ianjuta-editor.h>
 #include <libanjuta/interfaces/ianjuta-markable.h>
 #include <libanjuta/interfaces/ianjuta-symbol-manager.h>
-#include <libanjuta/plugins.h>
+#include <libanjuta/interfaces/ianjuta-preferences.h>
 #include <libegg/menu/egg-combo-action.h>
 
 #include <tm_tagmanager.h>
 #include "an_symbol_view.h"
 #include "an_symbol_search.h"
+#include "anjuta-symbol-locals.h"
 #include "an_symbol_info.h"
 #include "an_symbol_prefs.h"
 #include "an_symbol_iter.h"
@@ -45,7 +45,7 @@
 
 #define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta-symbol-browser-plugin.ui"
 #define PREFS_GLADE PACKAGE_DATA_DIR"/glade/anjuta-symbol-browser-plugin.glade"
-#define ICON_FILE "anjuta-symbol-browser-plugin.png"
+#define ICON_FILE "anjuta-symbol-browser-plugin-48.png"
 
 #define TIMEOUT_INTERVAL_SYMBOLS_UPDATE		10000
 
@@ -63,19 +63,14 @@ static void on_treesearch_symbol_selected_event (AnjutaSymbolSearch *search,
 												 SymbolBrowserPlugin *sv_plugin);
 static void update_editor_symbol_model (SymbolBrowserPlugin *sv_plugin);
 
-#define REGISTER_ICON(icon, stock_id) \
-	pixbuf = gdk_pixbuf_new_from_file (icon, NULL); \
-	icon_set = gtk_icon_set_new_from_pixbuf (pixbuf); \
-	gtk_icon_factory_add (icon_factory, stock_id, icon_set); \
-	g_object_unref (pixbuf);
+static void on_editor_update_ui (IAnjutaEditor *editor,
+								 SymbolBrowserPlugin *sv_plugin);
+static void on_char_added (IAnjutaEditor *editor, gint position, gchar ch,
+						   SymbolBrowserPlugin *sv_plugin);
 
 static void
 register_stock_icons (AnjutaPlugin *plugin)
 {
-	AnjutaUI *ui;
-	GtkIconFactory *icon_factory;
-	GtkIconSet *icon_set;
-	GdkPixbuf *pixbuf;
 	static gboolean registered = FALSE;
 
 	if (registered)
@@ -83,9 +78,9 @@ register_stock_icons (AnjutaPlugin *plugin)
 	registered = TRUE;
 
 	/* Register stock icons */
-	ui = anjuta_shell_get_ui (plugin->shell, NULL);
-	icon_factory = anjuta_ui_get_icon_factory (ui);
-	REGISTER_ICON (PACKAGE_PIXMAPS_DIR"/"ICON_FILE, "symbol-browser-plugin-icon");
+	BEGIN_REGISTER_ICON (plugin);
+	REGISTER_ICON (ICON_FILE, "symbol-browser-plugin-icon");
+	END_REGISTER_ICON;
 }
 
 static void
@@ -211,7 +206,7 @@ on_refresh_idle (gpointer user_data)
 	GList *source_uris;
 	GList *source_files;
 	AnjutaStatus *status;
-	SymbolBrowserPlugin *sv_plugin = (SymbolBrowserPlugin *)user_data;
+	SymbolBrowserPlugin *sv_plugin = ANJUTA_PLUGIN_SYMBOL_BROWSER (user_data);
 	
 	/* FIXME: There should be a way to ensure that this project manager
 	 * is indeed the one that has opened the project_uri
@@ -269,7 +264,7 @@ on_refresh_activate (GtkAction *action, SymbolBrowserPlugin *sv_plugin)
 	g_idle_add (on_refresh_idle, sv_plugin);
 }
 
-static GtkActionEntry popup_actions[] = 
+static GtkActionEntry actions[] = 
 {
 	{ "ActionMenuGoto", NULL, N_("_Goto"), NULL, NULL, NULL},
 	{
@@ -287,7 +282,11 @@ static GtkActionEntry popup_actions[] =
 		"<shift><control>d",
 		N_("Goto symbol declaration"),
 		G_CALLBACK (on_goto_file_tag_decl_activate)
-	},
+	}
+};
+
+static GtkActionEntry popup_actions[] = 
+{
 	{
 		"ActionPopupSymbolBrowserGotoDef",
 		NULL,
@@ -368,8 +367,8 @@ project_root_added (AnjutaPlugin *plugin, const gchar *name,
 	SymbolBrowserPlugin *sv_plugin;
 	const gchar *root_uri;
 
-	sv_plugin = (SymbolBrowserPlugin *)plugin;
-	
+	sv_plugin = ANJUTA_PLUGIN_SYMBOL_BROWSER (plugin);
+	  
 	g_free (sv_plugin->project_root_uri);
 	sv_plugin->project_root_uri = NULL;
 	root_uri = g_value_get_string (value);
@@ -383,10 +382,10 @@ project_root_added (AnjutaPlugin *plugin, const gchar *name,
 			trees_signals_block (sv_plugin);
 			anjuta_symbol_view_open (ANJUTA_SYMBOL_VIEW (sv_plugin->sv_tree),
 									 root_dir);
-			
+			 
 			/* Current editor symbol model may have changed */
 			update_editor_symbol_model (sv_plugin);
-			anjuta_status_progress_tick (status, NULL, _("Created sysmbols..."));
+			anjuta_status_progress_tick (status, NULL, _("Created symbols..."));
 			trees_signals_unblock (sv_plugin);
 			g_free (root_dir);
 		}
@@ -410,7 +409,7 @@ project_root_removed (AnjutaPlugin *plugin, const gchar *name,
 	IAnjutaProjectManager *pm;
 	SymbolBrowserPlugin *sv_plugin;
 	
-	sv_plugin = (SymbolBrowserPlugin *)plugin;
+	sv_plugin = ANJUTA_PLUGIN_SYMBOL_BROWSER (plugin);
 	
 	/* Disconnect events from project manager */
 	
@@ -473,7 +472,7 @@ on_treeview_event (GtkWidget *widget,
 		switch (e->keyval) {
 			case GDK_Return:
 				anjuta_ui_activate_action_by_group (sv_plugin->ui,
-													sv_plugin->action_group,
+													sv_plugin->popup_action_group,
 											"ActionPopupSymbolBrowserGotoDef");
 				return TRUE;
 			default:
@@ -496,7 +495,7 @@ on_treeview_row_activated (GtkTreeView *view, GtkTreePath *arg1,
 	if (!gtk_tree_selection_get_selected (selection, &model, &iter))
 		return;
 	anjuta_ui_activate_action_by_group (sv_plugin->ui,
-										sv_plugin->action_group,
+										sv_plugin->popup_action_group,
 										"ActionPopupSymbolBrowserGotoDef");
 }
 
@@ -523,6 +522,33 @@ trees_signals_unblock (SymbolBrowserPlugin *sv_plugin)
 									 NULL);
 }
 
+static void
+goto_tree_iter (SymbolBrowserPlugin *sv_plugin, GtkTreeIter *iter)
+{
+	gint line;
+	
+	/* FIXME: */
+	line = anjuta_symbol_view_workspace_get_line (ANJUTA_SYMBOL_VIEW
+												  (sv_plugin->sv_tree),
+												  iter);
+
+	if (line > 0 && sv_plugin->current_editor)
+	{
+		/* Goto line number */
+		ianjuta_editor_goto_line (IANJUTA_EDITOR (sv_plugin->current_editor),
+								  line, NULL);
+		if (IANJUTA_IS_MARKABLE (sv_plugin->current_editor))
+		{
+			ianjuta_markable_delete_all_markers (IANJUTA_MARKABLE (sv_plugin->current_editor),
+												 IANJUTA_MARKABLE_LINEMARKER,
+												 NULL);
+
+			ianjuta_markable_mark (IANJUTA_MARKABLE (sv_plugin->current_editor),
+								   line, IANJUTA_MARKABLE_LINEMARKER, NULL);
+		}
+	}
+}
+
 static void
 on_symbol_selected (GtkAction *action, SymbolBrowserPlugin *sv_plugin)
 {
@@ -530,29 +556,24 @@ on_symbol_selected (GtkAction *action, SymbolBrowserPlugin *sv_plugin)
 	
 	if (egg_combo_action_get_active_iter (EGG_COMBO_ACTION (action), &iter))
 	{
-		gint line;
-		line = anjuta_symbol_view_workspace_get_line (ANJUTA_SYMBOL_VIEW
-													  (sv_plugin->sv_tree),
-													  &iter);
-
-		if (line > 0 && sv_plugin->current_editor)
-		{
-			/* Goto line number */
-			ianjuta_editor_goto_line (IANJUTA_EDITOR (sv_plugin->current_editor),
-									  line, NULL);
-			if (IANJUTA_IS_MARKABLE (sv_plugin->current_editor))
-			{
-				ianjuta_markable_delete_all_markers (IANJUTA_MARKABLE (sv_plugin->current_editor),
-													 IANJUTA_MARKABLE_BASIC,
-													 NULL);
-
-				ianjuta_markable_mark (IANJUTA_MARKABLE (sv_plugin->current_editor),
-									   line, IANJUTA_MARKABLE_BASIC, NULL);
-			}
-		}
+		goto_tree_iter (sv_plugin, &iter);
 	}
 }
 
+static void
+on_local_treeview_row_activated (GtkTreeView *view, GtkTreePath *arg1,
+								 GtkTreeViewColumn *arg2,
+								 SymbolBrowserPlugin *sv_plugin)
+{
+	GtkTreeModel *model;
+	GtkTreeSelection *selection;
+	GtkTreeIter iter;
+
+	selection = gtk_tree_view_get_selection (view);
+	if (!gtk_tree_selection_get_selected (selection, &model, &iter))
+		return;
+	goto_tree_iter (sv_plugin, &iter);
+}
 
 /* -----------------------------------------------------------------------------
  * will manage the click of mouse and other events on search->hitlist treeview
@@ -574,7 +595,6 @@ on_treesearch_symbol_selected_event (AnjutaSymbolSearch *search,
 	}
 }
 
-
 static void
 on_editor_destroy (SymbolBrowserPlugin *sv_plugin, IAnjutaEditor *editor)
 {
@@ -592,7 +612,6 @@ on_editor_destroy (SymbolBrowserPlugin *sv_plugin, IAnjutaEditor *editor)
 	g_hash_table_remove (sv_plugin->editor_connected, G_OBJECT (editor));
 }
 
-
 static void
 on_editor_saved (IAnjutaEditor *editor, const gchar *saved_uri,
 				 SymbolBrowserPlugin *sv_plugin)
@@ -640,8 +659,14 @@ on_editor_saved (IAnjutaEditor *editor, const gchar *saved_uri,
 		g_object_set_data (G_OBJECT (editor), "tm_file",
 						   g_object_get_data (G_OBJECT (file_symbol_model),
 											  "tm_file"));
-		
+		/* Set toolbar version */
 		egg_combo_action_set_model (EGG_COMBO_ACTION (action), file_symbol_model);
+		/* Set local view version */
+		gtk_tree_view_set_model (GTK_TREE_VIEW (sv_plugin->sl_tree),
+								 file_symbol_model);
+		sv_plugin->locals_line_number = 0;
+		on_editor_update_ui (editor, sv_plugin);
+		
 		if (gtk_tree_model_iter_n_children (file_symbol_model, NULL) > 0)
 			g_object_set (G_OBJECT (action), "sensitive", TRUE, NULL);
 		else
@@ -653,7 +678,7 @@ on_editor_saved (IAnjutaEditor *editor, const gchar *saved_uri,
 		{
 			for (tmp = app->text_editor_list; tmp; tmp = g_list_next(tmp))
 			{
-				te1 = (TextEditor *) tmp->data;
+				te1 = TEXT_EDITOR (tmp->data);
 				text_editor_set_hilite_type(te1);
 			}
 		}
@@ -667,6 +692,12 @@ on_editor_foreach_disconnect (gpointer key, gpointer value, gpointer user_data)
 	g_signal_handlers_disconnect_by_func (G_OBJECT(key),
 										  G_CALLBACK (on_editor_saved),
 										  user_data);
+	g_signal_handlers_disconnect_by_func (G_OBJECT(key),
+										  G_CALLBACK (on_editor_update_ui),
+										  user_data);
+	g_signal_handlers_disconnect_by_func (G_OBJECT(key),
+										  G_CALLBACK (on_char_added),
+										  user_data);
 	g_object_weak_unref (G_OBJECT(key),
 						 (GWeakNotify) (on_editor_destroy),
 						 user_data);
@@ -676,7 +707,7 @@ static void
 on_editor_foreach_clear (gpointer key, gpointer value, gpointer user_data)
 {
 	const gchar *uri;
-	SymbolBrowserPlugin *sv_plugin = (SymbolBrowserPlugin *)user_data;
+	SymbolBrowserPlugin *sv_plugin = ANJUTA_PLUGIN_SYMBOL_BROWSER (user_data);
 	
 	uri = (const gchar *)value;
 	if (uri && strlen (uri) > 0)
@@ -713,22 +744,30 @@ update_editor_symbol_model (SymbolBrowserPlugin *sv_plugin)
 		anjuta_symbol_view_workspace_add_file (ANJUTA_SYMBOL_VIEW (sv_plugin->sv_tree), uri);
 		action = anjuta_ui_get_action (ui, "ActionGroupSymbolNavigation",
 									   "ActionGotoSymbol");
+		g_free (uri);
 		
 		file_symbol_model =
 			anjuta_symbol_view_get_file_symbol_model (ANJUTA_SYMBOL_VIEW (sv_plugin->sv_tree));
-		g_object_set_data (G_OBJECT (editor), "tm_file",
+		if (file_symbol_model)
+		{
+			g_object_set_data (G_OBJECT (editor), "tm_file",
 						   g_object_get_data (G_OBJECT (file_symbol_model),
 											  "tm_file"));
-		egg_combo_action_set_model (EGG_COMBO_ACTION (action), file_symbol_model);
-		
-		
-		if (gtk_tree_model_iter_n_children (file_symbol_model, NULL) > 0)
-			g_object_set (G_OBJECT (action), "sensitive", TRUE, NULL);
-		else
-			g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
+			/* Set toolbar version */
+			egg_combo_action_set_model (EGG_COMBO_ACTION (action), file_symbol_model);
+			
+			/* Set local view version */
+			gtk_tree_view_set_model (GTK_TREE_VIEW (sv_plugin->sl_tree),
+									 file_symbol_model);
+			sv_plugin->locals_line_number = 0;
+			on_editor_update_ui (IANJUTA_EDITOR (editor), sv_plugin);
+			
+			if (gtk_tree_model_iter_n_children (file_symbol_model, NULL) > 0)
+				g_object_set (G_OBJECT (action), "sensitive", TRUE, NULL);
+			else
+				g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
+		}
 	}
-	if (uri)
-		g_free (uri);
 }
 
 static gboolean
@@ -740,7 +779,7 @@ on_editor_buffer_symbols_update_timeout (gpointer user_data)
 	gint buffer_size = 0;
 	gchar *uri = NULL;
 
-	sv_plugin = (SymbolBrowserPlugin*) user_data;
+	sv_plugin = ANJUTA_PLUGIN_SYMBOL_BROWSER (user_data);
 	
 	if (sv_plugin->current_editor == NULL)
 		return FALSE;
@@ -754,7 +793,7 @@ on_editor_buffer_symbols_update_timeout (gpointer user_data)
 		ed = IANJUTA_EDITOR (sv_plugin->current_editor);
 		
 		buffer_size = ianjuta_editor_get_length (ed, NULL);
-		current_buffer = ianjuta_editor_get_text (ed, 0, buffer_size, NULL);
+		current_buffer = ianjuta_editor_get_text (ed, 0, -1, NULL);
 				
 		uri = ianjuta_file_get_uri (IANJUTA_FILE (ed), NULL);
 		
@@ -763,8 +802,11 @@ on_editor_buffer_symbols_update_timeout (gpointer user_data)
 		return FALSE;
 	
 	if (uri) {
+		/* FIXME: Only uncomment after investigating bug 395362 */
+		/*
 		anjuta_symbol_view_update_source_from_buffer (ANJUTA_SYMBOL_VIEW (sv_plugin->sv_tree), 
 				uri, current_buffer, buffer_size);
+		*/
 		g_free (uri);
 	}
 	
@@ -776,11 +818,62 @@ on_editor_buffer_symbols_update_timeout (gpointer user_data)
 	return TRUE;
 }
 
+static gboolean
+iter_matches (SymbolBrowserPlugin *sv_plugin, GtkTreeIter* iter, 
+			  GtkTreeModel* model, gint lineno)
+{
+	gint line;
+	gtk_tree_model_get (model, iter, COL_LINE, &line, -1);
+	if (line == lineno)
+	{
+		GtkTreePath* path = gtk_tree_model_get_path (model, iter);
+		GtkAction* action = anjuta_ui_get_action (sv_plugin->ui, 
+												  "ActionGroupSymbolNavigation",
+												  "ActionGotoSymbol");
+		
+		egg_combo_action_set_active_iter (EGG_COMBO_ACTION (action), iter);
+		gtk_tree_view_set_cursor (GTK_TREE_VIEW (sv_plugin->sl_tree), path, NULL,
+								 FALSE);
+		gtk_tree_path_free (path);
+		return TRUE;
+	}
+	return FALSE;
+}
+
+static void
+on_editor_update_ui (IAnjutaEditor *editor, SymbolBrowserPlugin *sv_plugin) 
+{
+	gint lineno = ianjuta_editor_get_lineno (editor, NULL);
+	
+	GtkTreeModel* model = anjuta_symbol_view_get_file_symbol_model
+							(ANJUTA_SYMBOL_VIEW(sv_plugin->sv_tree));
+	GtkTreeIter iter;
+	gboolean found = FALSE;
+	
+	if (sv_plugin->locals_line_number == lineno)
+		return;
+	sv_plugin->locals_line_number = lineno;
+	
+	if (!gtk_tree_model_get_iter_first (model, &iter))
+		return;
+	while (!found && lineno >= 0)
+	{
+		gtk_tree_model_get_iter_first (model, &iter);
+		do
+		{
+			found = iter_matches (sv_plugin, &iter, model, lineno);
+			if (found)
+				break;
+		}
+		while (gtk_tree_model_iter_next (model, &iter));
+		lineno--;
+	}
+}
+
 static void
 on_char_added (IAnjutaEditor *editor, gint position, gchar ch,
-				 SymbolBrowserPlugin *sv_plugin) {
-	
-	
+			   SymbolBrowserPlugin *sv_plugin)
+{
 	DEBUG_PRINT ("char added @ %d : %c [int %d]", position, ch, ch);
 	
 	/* try to force the update if a "." or a "->" is pressed */
@@ -802,7 +895,10 @@ value_added_current_editor (AnjutaPlugin *plugin, const char *name,
 	
 	editor = g_value_get_object (value);
 	
-	sv_plugin = (SymbolBrowserPlugin*)plugin;
+	if (!IANJUTA_IS_EDITOR(editor))
+		return;
+	
+	sv_plugin = ANJUTA_PLUGIN_SYMBOL_BROWSER (plugin);
 	
 	if (!sv_plugin->editor_connected)
 	{
@@ -822,8 +918,8 @@ value_added_current_editor (AnjutaPlugin *plugin, const char *name,
 						   sv_plugin);
 		if (uri)
 		{
-			g_hash_table_insert (sv_plugin->editor_connected, editor,
-								 g_strdup (uri));
+			g_hash_table_insert (sv_plugin->editor_connected, editor, uri);	//g_strdup (uri));
+			//g_free (uri);
 		}
 		else
 		{
@@ -837,11 +933,15 @@ value_added_current_editor (AnjutaPlugin *plugin, const char *name,
 		g_signal_connect (G_OBJECT (editor), "char-added",
 						  G_CALLBACK (on_char_added),
 						  sv_plugin);
+		g_signal_connect (G_OBJECT(editor), "update_ui",
+						  G_CALLBACK (on_editor_update_ui),
+						  sv_plugin);
 	}
-	g_free (uri);
 	
 	/* add a default timeout to the updating of buffer symbols */	
-	timeout_id = g_timeout_add (TIMEOUT_INTERVAL_SYMBOLS_UPDATE, on_editor_buffer_symbols_update_timeout, plugin);								 
+	timeout_id = g_timeout_add (TIMEOUT_INTERVAL_SYMBOLS_UPDATE,
+								on_editor_buffer_symbols_update_timeout,
+								plugin);
 	need_symbols_update = FALSE;
 	
 }
@@ -858,7 +958,7 @@ value_removed_current_editor (AnjutaPlugin *plugin,
 	g_source_remove (timeout_id);
 	need_symbols_update = FALSE;
 	
-	sv_plugin = (SymbolBrowserPlugin*)plugin;
+	sv_plugin = ANJUTA_PLUGIN_SYMBOL_BROWSER (plugin);
 	ui = anjuta_shell_get_ui (plugin->shell, NULL);
 	action = anjuta_ui_get_action (ui, "ActionGroupSymbolNavigation",
 								   "ActionGotoSymbol");
@@ -877,29 +977,47 @@ activate_plugin (AnjutaPlugin *plugin)
 	
 	register_stock_icons (plugin);
 	
-	sv_plugin = (SymbolBrowserPlugin*) plugin;
+	sv_plugin = ANJUTA_PLUGIN_SYMBOL_BROWSER (plugin);
 	sv_plugin->ui = anjuta_shell_get_ui (plugin->shell, NULL);
 	sv_plugin->prefs = anjuta_shell_get_preferences (plugin->shell, NULL);
 
 	/* Create widgets */
 	sv_plugin->sw = gtk_notebook_new();
-	
-	/* anjuta symbol view */
-	symbol_browser_prefs_init (sv_plugin);
 
-	/* create symbol-view scrolled window */
+	/* Local symbols */
+	sv_plugin->sl = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sv_plugin->sl),
+										 GTK_SHADOW_IN);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sv_plugin->sl),
+									GTK_POLICY_AUTOMATIC,
+									GTK_POLICY_AUTOMATIC);
+	
+	sv_plugin->sl_tab_label = gtk_label_new (_("Local" ));
+	sv_plugin->sl_tree = anjuta_symbol_locals_new ();
+	g_object_add_weak_pointer (G_OBJECT (sv_plugin->sl_tree),
+							   (gpointer)&sv_plugin->sl_tree);
+	g_signal_connect (G_OBJECT (sv_plugin->sl_tree), "row_activated",
+					  G_CALLBACK (on_local_treeview_row_activated), plugin);
+	gtk_container_add (GTK_CONTAINER(sv_plugin->sl), sv_plugin->sl_tree);
+	
+	/* add the scrolled window to the notebook */
+	gtk_notebook_append_page (GTK_NOTEBOOK(sv_plugin->sw),
+							  sv_plugin->sl, sv_plugin->sl_tab_label );
+
+	/* Global symbols */
 	sv_plugin->sv = gtk_scrolled_window_new (NULL, NULL);
-	sv_plugin->sv_tab_label = gtk_label_new (_("Tree" ));
-	/* setting up some properties */
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sv_plugin->sv),
 										 GTK_SHADOW_IN);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sv_plugin->sv),
 									GTK_POLICY_AUTOMATIC,
 									GTK_POLICY_AUTOMATIC);
 	
+	sv_plugin->sv_tab_label = gtk_label_new (_("Global" ));
 	sv_plugin->sv_tree = anjuta_symbol_view_new ();
+	symbol_browser_load_global_tags (plugin);
+	
 	g_object_add_weak_pointer (G_OBJECT (sv_plugin->sv_tree),
-							   (gpointer*)&sv_plugin->sv_tree);
+							   (gpointer)&sv_plugin->sv_tree);
 
 	g_signal_connect (G_OBJECT (sv_plugin->sv_tree), "event-after",
 					  G_CALLBACK (on_treeview_event), plugin);
@@ -907,18 +1025,18 @@ activate_plugin (AnjutaPlugin *plugin)
 					  G_CALLBACK (on_treeview_row_activated), plugin);
 
 	gtk_container_add (GTK_CONTAINER(sv_plugin->sv), sv_plugin->sv_tree);
+	
 	/* add the scrolled window to the notebook */
 	gtk_notebook_append_page (GTK_NOTEBOOK(sv_plugin->sw),
 							  sv_plugin->sv, sv_plugin->sv_tab_label );
 
 	/* anjuta symbol search */
-	
 	sv_plugin->ss =
 		anjuta_symbol_search_new (ANJUTA_SYMBOL_VIEW (sv_plugin->sv_tree));
 	sv_plugin->ss_tab_label = gtk_label_new (_("Search" ));
 
 	g_object_add_weak_pointer (G_OBJECT (sv_plugin->ss),
-							   (gpointer*)&sv_plugin->ss);
+							   (gpointer)&sv_plugin->ss);
 
 	gtk_notebook_append_page (GTK_NOTEBOOK(sv_plugin->sw), sv_plugin->ss,
 							  sv_plugin->ss_tab_label );
@@ -932,17 +1050,23 @@ activate_plugin (AnjutaPlugin *plugin)
 
 	
 	/* setting focus to the tree_view*/
-	gtk_widget_grab_focus(sv_plugin->sv_tree);
-	
+	gtk_notebook_set_current_page (GTK_NOTEBOOK (sv_plugin->sw), 0);
 	
 	/* Add action group */
 	sv_plugin->action_group = 
 		anjuta_ui_add_action_group_entries (sv_plugin->ui,
 											"ActionGroupSymbolBrowser",
+											_("Symbol browser actions"),
+											actions,
+											G_N_ELEMENTS (actions),
+											GETTEXT_PACKAGE, TRUE, plugin);
+	sv_plugin->popup_action_group = 
+		anjuta_ui_add_action_group_entries (sv_plugin->ui,
+											"ActionGroupPopupSymbolBrowser",
 											_("Symbol browser popup actions"),
 											popup_actions,
 											G_N_ELEMENTS (popup_actions),
-											GETTEXT_PACKAGE, plugin);
+											GETTEXT_PACKAGE, FALSE, plugin);
 	group = gtk_action_group_new ("ActionGroupSymbolNavigation");
 
 	/* create a new combobox in style of libegg... */
@@ -958,7 +1082,7 @@ activate_plugin (AnjutaPlugin *plugin)
 					  G_CALLBACK (on_symbol_selected), sv_plugin);
 	gtk_action_group_add_action (group, action);
 	anjuta_ui_add_action_group (sv_plugin->ui, "ActionGroupSymbolNavigation",
-								N_("Symbol navigations"), group);
+								N_("Symbol navigations"), group, FALSE);
 	sv_plugin->action_group_nav = group;
 	
 	/* Add UI */
@@ -980,7 +1104,6 @@ activate_plugin (AnjutaPlugin *plugin)
 		anjuta_plugin_add_watch (plugin, "document_manager_current_editor",
 								 value_added_current_editor,
 								 value_removed_current_editor, NULL);
-	
 	return TRUE;
 }
 
@@ -988,9 +1111,7 @@ static gboolean
 deactivate_plugin (AnjutaPlugin *plugin)
 {
 	SymbolBrowserPlugin *sv_plugin;
-	sv_plugin = (SymbolBrowserPlugin*) plugin;
-	
-	symbol_browser_prefs_finalize (sv_plugin);
+	sv_plugin = ANJUTA_PLUGIN_SYMBOL_BROWSER (plugin);
 	
 	/* Ensure all editor cached info are released */
 	if (sv_plugin->editor_connected)
@@ -1014,23 +1135,25 @@ deactivate_plugin (AnjutaPlugin *plugin)
 	
 	/* Remove action group */
 	anjuta_ui_remove_action_group (sv_plugin->ui, sv_plugin->action_group);
+	anjuta_ui_remove_action_group (sv_plugin->ui, sv_plugin->popup_action_group);
 	anjuta_ui_remove_action_group (sv_plugin->ui, sv_plugin->action_group_nav);
 	
 	sv_plugin->root_watch_id = 0;
 	sv_plugin->editor_watch_id = 0;
 	sv_plugin->merge_id = 0;
 	sv_plugin->sw = NULL;
+	sv_plugin->sl = NULL;
+	sv_plugin->sl_tree = NULL;
 	sv_plugin->sv = NULL;
 	sv_plugin->sv_tree = NULL;
 	sv_plugin->ss = NULL;
-	
 	return TRUE;
 }
 
 static void
 dispose (GObject *obj)
 {
-	SymbolBrowserPlugin *sv_plugin = (SymbolBrowserPlugin*) obj;
+	SymbolBrowserPlugin *sv_plugin = ANJUTA_PLUGIN_SYMBOL_BROWSER (obj);
 	/* Ensure all editors are disconnected */
 	if (sv_plugin->editor_connected)
 	{
@@ -1049,7 +1172,7 @@ dispose (GObject *obj)
 	*/
 	
 	g_object_remove_weak_pointer (G_OBJECT (sv_plugin->ss),
-										   (gpointer*)&sv_plugin->ss);
+										   (gpointer)&sv_plugin->ss);
 
 	
 	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (obj));
@@ -1058,19 +1181,21 @@ dispose (GObject *obj)
 static void
 finalize (GObject *obj)
 {
-	/* SymbolBrowserPlugin *plugin = (SymbolBrowserPlugin *) obj; */
+	/* SymbolBrowserPlugin *plugin = ANJUTA_PLUGIN_SYMBOL_BROWSER (obj); */
 	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (obj));
 }
 
 static void
 symbol_browser_plugin_instance_init (GObject *obj)
 {
-	SymbolBrowserPlugin *plugin = (SymbolBrowserPlugin*) obj;
+	SymbolBrowserPlugin *plugin = ANJUTA_PLUGIN_SYMBOL_BROWSER (obj);
 	plugin->current_editor = NULL;
 	plugin->editor_connected = NULL;
 	plugin->sw = NULL;
 	plugin->sv = NULL;
 	plugin->gconf_notify_ids = NULL;
+	plugin->locals_line_number = 0;
+	plugin->launcher = NULL;
 }
 
 static void
@@ -1080,8 +1205,8 @@ symbol_browser_plugin_class_init (SymbolBrowserPluginClass *klass)
 	AnjutaPluginClass *plugin_class;
 	
 	parent_class = g_type_class_peek_parent (klass);
-	object_class = (GObjectClass*) klass;
-	plugin_class = (AnjutaPluginClass*) klass;
+	object_class = G_OBJECT_CLASS (klass);
+	plugin_class = ANJUTA_PLUGIN_CLASS (klass);
 	
 	plugin_class->activate = activate_plugin;
 	plugin_class->deactivate = deactivate_plugin;
@@ -1109,7 +1234,7 @@ isymbol_manager_search (IAnjutaSymbolManager *sm,
 	
 	tags_array = tm_workspace_find (name, match_types, NULL,
 									partial_name_match, global_search);
-	if (tags_array)
+	if (tags_array && tags_array->len)
 	{
 		iter = anjuta_symbol_iter_new (tags_array);
 		return IANJUTA_ITERABLE (iter);
@@ -1127,10 +1252,10 @@ isymbol_manager_get_members (IAnjutaSymbolManager *sm,
 	AnjutaSymbolIter *iter = NULL;
 	
 	tags_array = tm_workspace_find_scope_members (NULL, symbol_name,
-												  global_search);
+												  global_search, TRUE);
 												  
 	
-	if (tags_array)
+	if (tags_array && tags_array->len)
 	{
 		iter = anjuta_symbol_iter_new (tags_array);
 		return IANJUTA_ITERABLE (iter);
@@ -1147,7 +1272,7 @@ isymbol_manager_get_parents (IAnjutaSymbolManager *sm,
 	AnjutaSymbolIter *iter = NULL;
 	
 	tags_array = tm_workspace_get_parents (symbol_name);
-	if (tags_array)
+	if (tags_array && tags_array->len)
 	{
 		iter = anjuta_symbol_iter_new (tags_array);
 		return IANJUTA_ITERABLE (iter);
@@ -1157,8 +1282,8 @@ isymbol_manager_get_parents (IAnjutaSymbolManager *sm,
 
 static IAnjutaIterable*
 isymbol_manager_get_completions_at_position (IAnjutaSymbolManager *sm,
-											gchar *file_uri,
-							 				gchar *text_buffer, 
+											const gchar *file_uri,
+							 				const gchar *text_buffer, 
 											gint text_length, 
 											gint text_pos,
 							 				GError **err)
@@ -1169,11 +1294,13 @@ isymbol_manager_get_completions_at_position (IAnjutaSymbolManager *sm,
 	IAnjutaEditor *ed;
 	AnjutaSymbolView *symbol_view;
 	gulong line;
+	gulong scope_position;
+	gchar *needed_text = NULL;
 	gint access_method;
 	GPtrArray * completable_tags_array;
 	AnjutaSymbolIter *iter = NULL;
 	
-	sv_plugin = (SymbolBrowserPlugin*)sm;
+	sv_plugin = ANJUTA_PLUGIN_SYMBOL_BROWSER (sm);
 	ed = IANJUTA_EDITOR (sv_plugin->current_editor);	
 	symbol_view = ANJUTA_SYMBOL_VIEW (sv_plugin->sv_tree);
 	
@@ -1185,14 +1312,49 @@ isymbol_manager_get_completions_at_position (IAnjutaSymbolManager *sm,
 	/* check whether the current file_uri is listed in the tm_workspace or not... */	
 	if (tm_file == NULL)
 		return 	NULL;
-	func_scope_tag = tm_get_current_function (tm_file->work_object.tags_array, line);
-	
 		
+
+	// FIXME: remove DEBUG_PRINT	
+/*/
+	DEBUG_PRINT ("tags in file &s\n");
+	if (tm_file->work_object.tags_array != NULL) {
+		int i;
+		for (i=0; i < tm_file->work_object.tags_array->len; i++) {
+			TMTag *cur_tag;
+		
+			cur_tag = (TMTag*)g_ptr_array_index (tm_file->work_object.tags_array, i);
+			tm_tag_print (cur_tag, stdout);
+		}
+	}
+/*/
+
+	func_scope_tag = tm_get_current_function (tm_file->work_object.tags_array, line);
+		
+	if (func_scope_tag == NULL) {
+		DEBUG_PRINT ("func_scope_tag is NULL, seems like it's a completion on a global scope");
+		return NULL;
+	}
+	
+	DEBUG_PRINT ("current expression scope: %s", func_scope_tag->name);
+	
+	
+	scope_position = ianjuta_editor_get_line_begin_position (ed, func_scope_tag->atts.entry.line, NULL);
+	needed_text = ianjuta_editor_get_text (ed, scope_position,
+										   text_pos - scope_position, NULL);
+
+	if (needed_text == NULL)
+		DEBUG_PRINT ("needed_text is null");
+	DEBUG_PRINT ("text needed is %s", needed_text );
+	
+
+	/* we'll pass only the text of the current scope: i.e. only the current function
+	 * in which we request the completion. */
 	TMTag * found_type = anjuta_symbol_view_get_type_of_expression (symbol_view, 
-				text_buffer, text_pos, func_scope_tag, &access_method);
+				needed_text, text_pos - scope_position, func_scope_tag, &access_method);
 
 				
 	if (found_type == NULL) {
+		DEBUG_PRINT ("type not found.");
 		return NULL;	
 	}
 	
@@ -1204,7 +1366,7 @@ isymbol_manager_get_completions_at_position (IAnjutaSymbolManager *sm,
 	else
 		completable_tags_array = anjuta_symbol_view_get_completable_members (found_type, TRUE);
 	
-	if (completable_tags_array)
+	if (completable_tags_array && completable_tags_array->len)
 	{
 		iter = anjuta_symbol_iter_new (completable_tags_array);
 		return IANJUTA_ITERABLE (iter);
@@ -1212,7 +1374,6 @@ isymbol_manager_get_completions_at_position (IAnjutaSymbolManager *sm,
 	return NULL;
 }
 
-
 static void
 isymbol_manager_iface_init (IAnjutaSymbolManagerIface *iface)
 {
@@ -1222,8 +1383,28 @@ isymbol_manager_iface_init (IAnjutaSymbolManagerIface *iface)
 	iface->get_completions_at_position = isymbol_manager_get_completions_at_position;
 }
 
+static void
+ipreferences_merge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError** e)
+{
+	symbol_browser_prefs_init(ANJUTA_PLUGIN_SYMBOL_BROWSER (ipref));
+}
+
+static void
+ipreferences_unmerge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError** e)
+{
+	symbol_browser_prefs_finalize (ANJUTA_PLUGIN_SYMBOL_BROWSER (ipref));
+}
+
+static void
+ipreferences_iface_init(IAnjutaPreferencesIface* iface)
+{
+	iface->merge = ipreferences_merge;
+	iface->unmerge = ipreferences_unmerge;	
+}
+
 ANJUTA_PLUGIN_BEGIN (SymbolBrowserPlugin, symbol_browser_plugin);
 ANJUTA_PLUGIN_ADD_INTERFACE (isymbol_manager, IANJUTA_TYPE_SYMBOL_MANAGER);
+ANJUTA_PLUGIN_ADD_INTERFACE (ipreferences, IANJUTA_TYPE_PREFERENCES);
 ANJUTA_PLUGIN_END;
 
 ANJUTA_SIMPLE_PLUGIN (SymbolBrowserPlugin, symbol_browser_plugin);
