@@ -15,20 +15,24 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include <config.h>
+
+#include <signal.h>
+
 #include <libanjuta/anjuta-shell.h>
-#include <libanjuta/anjuta-children.h>
 #include <libanjuta/anjuta-debug.h>
 
 #include <libanjuta/interfaces/ianjuta-terminal.h>
-#include <libanjuta/plugins.h>
+#include <libanjuta/interfaces/ianjuta-preferences.h>
+
+#include <signal.h>
 
 #define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta-terminal.ui"
 #define PREFS_GLADE PACKAGE_DATA_DIR"/glade/anjuta-terminal-plugin.glade"
-#define ICON_FILE "preferences-terminal.png"
+#define ICON_FILE "anjuta-terminal-plugin-48.png"
 
 /* Some desktop/gnome-terminal gconf keys. */
 #define GCONF_MONOSPACE_FONT      "/desktop/gnome/interface/monospace_font_name"
@@ -61,6 +65,14 @@
 
 extern char **environ;
 
+extern GType terminal_plugin_get_type (GTypeModule *module);
+#define ANJUTA_PLUGIN_TERMINAL_TYPE         (terminal_plugin_get_type (NULL))
+#define ANJUTA_PLUGIN_TERMINAL(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), ANJUTA_PLUGIN_TERMINAL_TYPE, TerminalPlugin))
+#define ANJUTA_PLUGIN_TERMINAL_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST ((k), ANJUTA_PLUGIN_TERMINAL_CLASS, TerminalPluginClass))
+#define ANJUTA_IS_PLUGIN_TERMINAL(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), ANJUTA_PLUGIN_TERMINAL_TYPE))
+#define ANJUTA_IS_PLUGIN_TERMINAL_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), ANJUTA_PLUGIN_TERMINAL_TYPE))
+#define ANJUTA_PLUGIN_TERMINAL_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), ANJUTA_PLUGIN_TERMINAL_TYPE, TerminalPluginClass))
+
 typedef struct _TerminalPlugin TerminalPlugin;
 typedef struct _TerminalPluginClass TerminalPluginClass;
 
@@ -78,7 +90,9 @@ struct _TerminalPlugin{
 	GtkWidget *pref_default_button;
 	GList *gconf_notify_ids;
 	gboolean child_initizlized;
+#if OLD_VTE == 1
 	gboolean first_time_realization;
+#endif
 };
 
 struct _TerminalPluginClass{
@@ -136,33 +150,40 @@ preferences_changed (AnjutaPreferences *prefs, TerminalPlugin *term)
 											  GCONF_VALUE_STRING, NULL);
 	if (profiles)
 	{
-		GList *list = NULL;
-		GSList *node = profiles;
-		while (node)
+		if (term->pref_profile_combo)
 		{
-			if (node->data)
-				list = g_list_append (list, node->data);
-			node = g_slist_next (node);
+			GList *list = NULL;
+			GSList *node = profiles;
+			while (node)
+			{
+				if (node->data)
+					list = g_list_append (list, node->data);
+				node = g_slist_next (node);
+			}
+			gtk_combo_set_popdown_strings (GTK_COMBO (term->pref_profile_combo),
+										   list);
+			g_list_free (list);
 		}
-		gtk_combo_set_popdown_strings (GTK_COMBO (term->pref_profile_combo), list);
-		g_list_free (list);
+		g_slist_foreach (profiles, (GFunc)g_free, NULL);
+		g_slist_free (profiles);
 	}
-	setting = anjuta_preferences_get_int (pref, PREFS_TERMINAL_PROFILE_USE_DEFAULT);
+	setting = anjuta_preferences_get_int (pref,
+										  PREFS_TERMINAL_PROFILE_USE_DEFAULT);
 	if (setting)
 	{
 		/* Use the currently selected profile in gnome-terminal */
 		text = gconf_client_get_string (client, GCONF_DEFAULT_PROFILE, NULL);
 		if (!text)
-			text = "Default";
+			text = g_strdup ("Default");
 	}
 	else
 	{
 		/* Otherwise use the user selected profile */
 		text = anjuta_preferences_get (pref, PREFS_TERMINAL_PROFILE);
 		if (!text)
-			text = "Default";
+			text = g_strdup ("Default");
 	}
-	profile = g_strdup (text);
+	profile = text;
 	
 	vte_terminal_set_mouse_autohide (VTE_TERMINAL (vte), TRUE);
 
@@ -177,7 +198,8 @@ preferences_changed (AnjutaPreferences *prefs, TerminalPlugin *term)
 	}
 	if (text && GTK_WIDGET (vte)->window)
 		vte_terminal_set_font_from_string (VTE_TERMINAL (vte), text);
-
+	g_free (text);
+	
 	setting = GET_PROFILE_BOOL (GCONF_CURSOR_BLINK);
 	vte_terminal_set_cursor_blinks (VTE_TERMINAL (vte), setting);
 	setting = GET_PROFILE_BOOL (GCONF_SILENT_BELL);
@@ -191,7 +213,8 @@ preferences_changed (AnjutaPreferences *prefs, TerminalPlugin *term)
 	text = GET_PROFILE_STRING (GCONF_WORD_CHARS);
 	if (text)
 		vte_terminal_set_word_chars (VTE_TERMINAL (vte), text);
-
+	g_free (text);
+	
 	text = GET_PROFILE_STRING (GCONF_BACKSPACE_BINDING);
 	if (text)
 	{
@@ -207,6 +230,7 @@ preferences_changed (AnjutaPreferences *prefs, TerminalPlugin *term)
 		else
 			vte_terminal_set_backspace_binding (VTE_TERMINAL (vte),
 								VTE_ERASE_AUTO);
+		g_free (text);
 	}
 	text = GET_PROFILE_STRING (GCONF_DELETE_BINDING);
 	if (text)
@@ -223,6 +247,7 @@ preferences_changed (AnjutaPreferences *prefs, TerminalPlugin *term)
 		else
 			vte_terminal_set_delete_binding (VTE_TERMINAL (vte),
 							 VTE_ERASE_AUTO);
+		g_free (text);
 	}
 	/* Set fore- and background colors. */
 	text = GET_PROFILE_STRING (GCONF_BACKGROUND_COLOR);
@@ -230,6 +255,7 @@ preferences_changed (AnjutaPreferences *prefs, TerminalPlugin *term)
 	{
 		gdk_color_parse (text, &color);
 		vte_terminal_set_color_background (VTE_TERMINAL (vte), &color);
+		g_free (text);
 	}
 	text = GET_PROFILE_STRING (GCONF_FOREGROUND_COLOR);
 	if (text)
@@ -237,6 +263,7 @@ preferences_changed (AnjutaPreferences *prefs, TerminalPlugin *term)
 		gdk_color_parse (text, &color);
 		vte_terminal_set_color_foreground (VTE_TERMINAL (vte), &color);
 		vte_terminal_set_color_bold (VTE_TERMINAL (vte), &color);
+		g_free (text);
 	}
 	g_free (profile);
 	g_object_unref (client);
@@ -246,7 +273,7 @@ static void
 on_gconf_notify_prefs (GConfClient *gclient, guint cnxn_id,
 					   GConfEntry *entry, gpointer user_data)
 {
-	TerminalPlugin *tp = (TerminalPlugin*)user_data;
+	TerminalPlugin *tp = ANJUTA_PLUGIN_TERMINAL (user_data);
 	preferences_changed (tp->prefs, tp);
 }
 
@@ -254,7 +281,7 @@ on_gconf_notify_prefs (GConfClient *gclient, guint cnxn_id,
 	notify_id = anjuta_preferences_notify_add (tp->prefs, \
 											   key, func, tp, NULL); \
 	tp->gconf_notify_ids = g_list_prepend (tp->gconf_notify_ids, \
-										   (gpointer)(notify_id));
+										   GUINT_TO_POINTER (notify_id));
 static void
 prefs_init (TerminalPlugin *tp)
 {
@@ -270,7 +297,8 @@ prefs_finalize (TerminalPlugin *tp)
 	node = tp->gconf_notify_ids;
 	while (node)
 	{
-		anjuta_preferences_notify_remove (tp->prefs, (guint)node->data);
+		anjuta_preferences_notify_remove (tp->prefs,
+										  GPOINTER_TO_UINT (node->data));
 		node = g_list_next (node);
 	}
 	g_list_free (tp->gconf_notify_ids);
@@ -358,24 +386,14 @@ terminal_execute (TerminalPlugin *term_plugin, const gchar *directory,
 
 	env = get_child_environment ();
 	
-#if OLD_VTE == 1
-	if (dir)
-		chdir (dir);
-	term_plugin->child_pid = vte_terminal_fork_command (term, args[0],
-														args, env);
-#else
 	term_plugin->child_pid = vte_terminal_fork_command (term, args[0], args,
 														env, dir, 0, 0, 0);
-#endif
 	g_free (dir);
 	g_strfreev (env);
 	g_free (args);
 	g_list_foreach (args_list, (GFunc)g_free, NULL);
 	g_list_free (args_list);
 	
-	/* The fork command above overwirtes our SIGCHLD signal handler.
-	 * Restore it */
-	anjuta_children_recover ();
 	preferences_changed (term_plugin->prefs, term_plugin);
 	
 	anjuta_shell_present_widget (ANJUTA_PLUGIN (term_plugin)->shell,
@@ -449,6 +467,7 @@ terminal_keypress_cb (GtkWidget *widget, GdkEventKey  *event,
 	return FALSE;
 }
 
+#if OLD_VTE == 1
 /* VTE has a terrible bug where it could crash when container is changed.
  * The problem has been traced in vte where its style-set handler does not
  * adequately check if the widget is realized, resulting in a crash when
@@ -491,6 +510,7 @@ terminal_unrealize_cb (GtkWidget *term, TerminalPlugin *plugin)
 											 NULL);
 	DEBUG_PRINT ("Blocked %d terminal signal", count);
 }
+#endif
 
 static void
 terminal_destroy_cb (GtkWidget *widget, TerminalPlugin *term)
@@ -521,11 +541,12 @@ terminal_create (TerminalPlugin *term_plugin)
 					  G_CALLBACK (terminal_destroy_cb), term_plugin);
 	g_signal_connect (G_OBJECT (term_plugin->term), "event",
 					  G_CALLBACK (terminal_keypress_cb), term_plugin);
+#if OLD_VTE == 1
 	g_signal_connect (G_OBJECT (term_plugin->term), "realize",
 					  G_CALLBACK (terminal_realize_cb), term_plugin);
 	g_signal_connect (G_OBJECT (term_plugin->term), "unrealize",
 					  G_CALLBACK (terminal_unrealize_cb), term_plugin);
-
+#endif
 	sb = gtk_vscrollbar_new (GTK_ADJUSTMENT (VTE_TERMINAL (term_plugin->term)->adjustment));
 	GTK_WIDGET_UNSET_FLAGS (sb, GTK_CAN_FOCUS);
 	
@@ -544,41 +565,29 @@ terminal_create (TerminalPlugin *term_plugin)
 	term_plugin->hbox = hbox;
 }
 
-#define REGISTER_ICON(icon, stock_id) \
-	pixbuf = gdk_pixbuf_new_from_file (icon, NULL); \
-	icon_set = gtk_icon_set_new_from_pixbuf (pixbuf); \
-	gtk_icon_factory_add (icon_factory, stock_id, icon_set); \
-	g_object_unref (pixbuf);
-
 static void
 register_stock_icons (AnjutaPlugin *plugin)
 {
-	AnjutaUI *ui;
-	GtkIconFactory *icon_factory;
-	GtkIconSet *icon_set;
-	GdkPixbuf *pixbuf;
 	static gboolean registered = FALSE;
 
 	if (registered)
 		return;
 	registered = TRUE;
 
-	/* Register stock icons */
-	ui = anjuta_shell_get_ui (plugin->shell, NULL);
-	icon_factory = anjuta_ui_get_icon_factory (ui);
-	REGISTER_ICON (PACKAGE_PIXMAPS_DIR"/"ICON_FILE, "terminal-plugin-icon");
+	BEGIN_REGISTER_ICON (plugin);
+	REGISTER_ICON (ICON_FILE, "terminal-plugin-icon");
+	END_REGISTER_ICON;
 }
 
 static gboolean
 activate_plugin (AnjutaPlugin *plugin)
 {
-	GladeXML *gxml;
 	TerminalPlugin *term_plugin;
 	static gboolean initialized = FALSE;
 	
 	DEBUG_PRINT ("TerminalPlugin: Activating Terminal plugin ...");
 	
-	term_plugin = (TerminalPlugin*) plugin;
+	term_plugin = ANJUTA_PLUGIN_TERMINAL (plugin);
 	term_plugin->ui = anjuta_shell_get_ui (plugin->shell, NULL);
 	term_plugin->prefs = anjuta_shell_get_preferences (plugin->shell, NULL);
 	
@@ -587,18 +596,6 @@ activate_plugin (AnjutaPlugin *plugin)
 	if (!initialized)
 	{
 		register_stock_icons (plugin);
-		
-		/* Create the terminal preferences page */
-		gxml = glade_xml_new (PREFS_GLADE, "preferences_dialog_terminal", NULL);
-		anjuta_preferences_add_page (term_plugin->prefs, gxml,
-									"Terminal", ICON_FILE);
-		term_plugin->pref_profile_combo = glade_xml_get_widget (gxml, "profile_list_combo");
-		term_plugin->pref_default_button =
-			glade_xml_get_widget (gxml,
-						"preferences_toggle:bool:1:0:terminal.default.profile");
-		g_signal_connect (G_OBJECT(term_plugin->pref_default_button), "toggled",
-						  G_CALLBACK (use_default_profile_cb), term_plugin);
-		g_object_unref (gxml);
 	}
 	
 	/* Setup prefs callbacks */
@@ -619,10 +616,15 @@ static gboolean
 deactivate_plugin (AnjutaPlugin *plugin)
 {
 	TerminalPlugin *term_plugin;
-	term_plugin = (TerminalPlugin*) plugin;
+	term_plugin = ANJUTA_PLUGIN_TERMINAL (plugin);
 	
 	prefs_finalize (term_plugin);
-	
+
+#if OLD_VTE == 1
+	g_signal_handlers_disconnect_by_func (G_OBJECT (term_plugin->term),
+			 G_CALLBACK (terminal_unrealize_cb), term_plugin);
+#endif
+
 	/* terminal plugin widgets are destroyed as soon as it is removed */
 	anjuta_shell_remove_widget (plugin->shell, term_plugin->frame, NULL);
 	
@@ -636,7 +638,9 @@ deactivate_plugin (AnjutaPlugin *plugin)
 	term_plugin->scrollbar = NULL;
 	term_plugin->hbox = NULL;
 	term_plugin->child_initizlized = FALSE;
+#if OLD_VTE == 1
 	term_plugin->first_time_realization = TRUE;
+#endif
 
 	// terminal_finalize (term_plugin);
 	return TRUE;
@@ -657,10 +661,13 @@ terminal_plugin_finalize (GObject *obj)
 static void
 terminal_plugin_instance_init (GObject *obj)
 {
-	TerminalPlugin *plugin = (TerminalPlugin*) obj;
+	TerminalPlugin *plugin = ANJUTA_PLUGIN_TERMINAL (obj);
 	plugin->gconf_notify_ids = NULL;
 	plugin->child_initizlized = FALSE;
+	plugin->pref_profile_combo = NULL;
+#if OLD_VTE == 1
 	plugin->first_time_realization = TRUE;
+#endif
 }
 
 static void
@@ -684,7 +691,7 @@ iterminal_execute_command (IAnjutaTerminal *terminal,
 	TerminalPlugin *plugin;
 	const gchar *dir;
 	
-	plugin = (TerminalPlugin*)G_OBJECT (terminal);
+	plugin = ANJUTA_PLUGIN_TERMINAL (terminal);
 	
 	if (directory == NULL || strlen (directory) <= 0)
 		dir = NULL;
@@ -700,8 +707,43 @@ iterminal_iface_init(IAnjutaTerminalIface *iface)
 	iface->execute_command = iterminal_execute_command;
 }
 
+static void
+ipreferences_merge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError** e)
+{
+	/* Create the terminal preferences page */
+	TerminalPlugin* term_plugin = ANJUTA_PLUGIN_TERMINAL (ipref);
+	GladeXML *gxml = glade_xml_new (PREFS_GLADE, "preferences_dialog_terminal", NULL);
+	anjuta_preferences_add_page (term_plugin->prefs, gxml,
+									"Terminal", _("Terminal"), ICON_FILE);
+	term_plugin->pref_profile_combo = glade_xml_get_widget (gxml, "profile_list_combo");
+	term_plugin->pref_default_button =
+		glade_xml_get_widget (gxml,
+						"preferences_toggle:bool:1:0:terminal.default.profile");
+	g_signal_connect (G_OBJECT(term_plugin->pref_default_button), "toggled",
+						  G_CALLBACK (use_default_profile_cb), term_plugin);
+	g_object_unref (gxml);
+}
+
+static void
+ipreferences_unmerge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError** e)
+{
+	TerminalPlugin* term_plugin = ANJUTA_PLUGIN_TERMINAL (ipref);
+	g_signal_handlers_disconnect_by_func(G_OBJECT(term_plugin->pref_default_button),
+		G_CALLBACK (use_default_profile_cb), term_plugin);
+	anjuta_preferences_remove_page(prefs, _("Terminal"));
+	term_plugin->pref_profile_combo = NULL;
+}
+
+static void
+ipreferences_iface_init(IAnjutaPreferencesIface* iface)
+{
+	iface->merge = ipreferences_merge;
+	iface->unmerge = ipreferences_unmerge;	
+}
+
 ANJUTA_PLUGIN_BEGIN (TerminalPlugin, terminal_plugin);
 ANJUTA_PLUGIN_ADD_INTERFACE (iterminal, IANJUTA_TYPE_TERMINAL);
+ANJUTA_PLUGIN_ADD_INTERFACE (ipreferences, IANJUTA_TYPE_PREFERENCES);
 ANJUTA_PLUGIN_END;
 
 ANJUTA_SIMPLE_PLUGIN (TerminalPlugin, terminal_plugin);
